@@ -21,6 +21,14 @@
 #' @param trace_batch_size the number of posterior samples to process at a time
 #'   when `target` is a `greta_mcmc_list` object; reduce this to
 #'   reduce memory demands
+#' @param compute_options Default is to use CPU only with `cpu_only()`. Use
+#'   `gpu_only()` to use only GPU. In the future we will add more options for
+#'   specifying CPU and GPU use.  If setting GPU with `gpu_only()` then we
+#'   cannot always guarantee that the random number seed will be respected. This
+#'   is due to the way tensorflow interfaces with the GPU. If you must have
+#'   reproducibility of all simulations we recommend using `cpu_only()`, which
+#'   is the default. You can turn off the message about setting seed with GPU
+#'   usage using `options(greta_gpu_message = FALSE)`
 #'
 #' @return Values of the target greta array(s), given values of the greta arrays
 #'   on which they depend (either specified in `values` or sampled from
@@ -141,108 +149,131 @@
 #'   trace_batch_size = Inf
 #' )
 #' }
-calculate <- function(...,
-                      values = list(),
-                      nsim = NULL,
-                      seed = NULL,
-                      precision = c("double", "single"),
-                      trace_batch_size = 100) {
+calculate <- function(
+  ...,
+  values = list(),
+  nsim = NULL,
+  seed = NULL,
+  precision = c("double", "single"),
+  trace_batch_size = 100,
+  compute_options = cpu_only()
+) {
 
-  # turn the provided greta arrays into a list and try to find the names
-  target <- list(...)
-  names <- names(target)
+  # set device to be CPU/GPU for the entire run
+  with(tf$device(compute_options), {
 
-  # see if any names are missing and try to fill them in
-  if (is.null(names)) {
-    names_missing <- rep(TRUE, length(target))
-  } else {
-    names_missing <- names == ""
-  }
+    # message users about random seeds and GPU usage if they are using GPU
+    message_if_using_gpu(compute_options)
 
-  if (any(names_missing)) {
-    scraped_names <- substitute(list(...))[-1]
-    missing_names <- vapply(scraped_names[names_missing], deparse, "")
-    names[names_missing] <- missing_names
-    names(target) <- names
-  }
+    # turn the provided greta arrays into a list
+    target <- list(...)
 
-  # catch empty lists here, since check_greta_arrays assumes data greta arrays
-  # have been stripped out
-  if (identical(target, list())) {
-    msg <- cli::format_error(
-      c(
-        "{.fun calculate} requires {.cls greta array}s",
-        "no {.cls greta array}s were provided to {.fun calculate}"
-      )
+    # try to find the names
+    # REFACTOR as: target <- find_and_name_arrays(target)
+    names <- names(target)
+
+    # see if any names are missing and try to fill them in
+    if (is.null(names)) {
+      names_missing <- rep(TRUE, length(target))
+    } else {
+      names_missing <- names == ""
+    }
+
+    if (any(names_missing)) {
+      scraped_names <- substitute(list(...))[-1]
+      missing_names <- vapply(scraped_names[names_missing], deparse, "")
+      names[names_missing] <- missing_names
+      names(target) <- names
+    }
+
+    # catch empty lists here, since check_greta_arrays assumes data greta arrays
+    # have been stripped out
+    check_if_array_is_empty_list(target)
+
+    # check the inputs
+    check_greta_arrays(
+      target,
+      "calculate",
+      "Perhaps you forgot to explicitly name other arguments?"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
 
-  # check the inputs
-  check_greta_arrays(
-    target,
-    "calculate",
-    "Perhaps you forgot to explicitly name other arguments?"
-  )
+    # checks and RNG seed setting if we're sampling
+    # REFACTOR: check_rng_seed(nim, seed, compute_option)
+    if (!is.null(nsim)) {
 
-  # checks and RNG seed setting if we're sampling
-  if (!is.null(nsim)) {
+      # check nsim is valid
+      nsim <- check_positive_integer(nsim, "nsim")
 
-    # check nsim is valid
-    nsim <- check_positive_integer(nsim, "nsim")
+      # if an RNG seed was provided use it and reset the RNG on exiting
+      if (!is.null(seed)) {
+        no_global_random_seed <- !exists(
+          x = ".Random.seed",
+          envir = .GlobalEnv,
+          inherits = FALSE
+          )
+        if (no_global_random_seed) {
+          runif(1)
+        }
 
-    # if an RNG seed was provided use it and reset the RNG on exiting
-    if (!is.null(seed)) {
-      if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
-        runif(1)
+
+        r_seed <- get(".Random.seed", envir = .GlobalEnv)
+        on.exit(assign(".Random.seed", r_seed, envir = .GlobalEnv))
+        tensorflow::set_random_seed(
+          seed = seed,
+          disable_gpu = is_using_cpu(compute_options)
+          )
+      }
+
+      if (is.null(seed)){
+        tensorflow::set_random_seed(
+          seed = get_seed(),
+          disable_gpu = is_using_cpu(compute_options)
+        )
       }
 
-      r_seed <- get(".Random.seed", envir = .GlobalEnv)
-      on.exit(assign(".Random.seed", r_seed, envir = .GlobalEnv))
-      set.seed(seed)
     }
-  }
 
-  # set precision
-  tf_float <- switch(match.arg(precision),
-    double = "float64",
-    single = "float32"
-  )
-
-  if (inherits(values, "greta_mcmc_list")) {
-    result <- calculate_greta_mcmc_list(
-      target = target,
-      values = values,
-      nsim = nsim,
-      tf_float = tf_float,
-      trace_batch_size = trace_batch_size
+    # set precision
+    tf_float <- switch(
+      match.arg(precision),
+      double = "float64",
+      single = "float32"
     )
-  } else {
-    result <- calculate_list(
-      target = target,
-      values = values,
-      nsim = nsim,
-      tf_float = tf_float,
-      env = parent.frame()
-    )
-  }
 
-  if (!inherits(result, "greta_mcmc_list")) {
-
-    # if it's not mcmc samples, make sure the results are in the right order
-    # (tensorflow order seems to be platform specific?!?)
-    order <- match(names(result), names(target))
-    result <- result[order]
-
-    # if the result wasn't mcmc samples or simulations, drop the batch dimension
-    if (is.null(nsim)) {
-      result <- lapply(result, drop_first_dim)
+    if (is.greta_mcmc_list(values)) {
+      result <- calculate_greta_mcmc_list(
+        target = target,
+        values = values,
+        nsim = nsim,
+        tf_float = tf_float,
+        trace_batch_size = trace_batch_size
+      )
+    } else {
+      result <- calculate_list(
+        target = target,
+        values = values,
+        nsim = nsim,
+        tf_float = tf_float,
+        env = parent.frame()
+      )
     }
-  }
-  result
+
+    if (!is.greta_mcmc_list(result)) {
+
+      # if it's not mcmc samples, make sure the results are in the right order
+      # (tensorflow order seems to be platform specific?!?)
+      order <- match(names(result), names(target))
+      result <- result[order]
+
+      # if the result wasn't mcmc samples or simulations, drop the batch dimension
+      if (is.null(nsim)) {
+        result <- lapply(result, drop_first_dim)
+      }
+    }
+    result
+
+    # close tf$device call to use CPU or GPU
+  })
 }
 
 #' @importFrom coda thin
@@ -252,6 +283,8 @@ calculate_greta_mcmc_list <- function(target,
                                       nsim,
                                       tf_float,
                                       trace_batch_size) {
+
+  # assign the free state
   stochastic <- !is.null(nsim)
 
   # check trace_batch_size is valid
@@ -260,6 +293,8 @@ calculate_greta_mcmc_list <- function(target,
   # get the free state draws and old dag from the samples
   model_info <- get_model_info(values)
   mcmc_dag <- model_info$model$dag
+
+  # this is the free state MCMC object
   draws <- model_info$raw_draws
 
   # build a new dag from the targets
@@ -282,83 +317,27 @@ calculate_greta_mcmc_list <- function(target,
   dag$variables_without_free_state <- dag_variables[stateless_names]
 
   # check there's some commonality between the two dags
-  connected_to_draws <- names(dag$node_list) %in% names(mcmc_dag$node_list)
-  if (!any(connected_to_draws)) {
-    msg <- cli::format_error(
-      "the target {.cls greta array}s do not appear to be connected to those \\
-      in the {.cls greta_mcmc_list} object"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_commanality_btn_dags(dag, mcmc_dag)
 
   # if they didn't specify nsim, check we can deterministically compute the
   # targets from the draws
   if (!stochastic) {
 
     # see if the new dag introduces any new variables
-    new_types <- dag$node_types[!connected_to_draws]
-    if (any(new_types == "variable")) {
-      msg <- cli::format_error(
-        c(
-          "{.arg nsim} must be set to sample {.cls greta array}s not in MCMC \\
-          samples",
-          "the target {.cls greta array}s are related to new variables that \\
-          are not in the MCMC samples, so cannot be calculated from the \\
-          samples alone.",
-          "Set {.arg nsim} if you want to sample them conditionally on the \\
-          MCMC samples"
-        )
-      )
-      stop(
-        msg,
-        call. = FALSE
-      )
-    }
+    check_dag_introduces_new_variables(dag, mcmc_dag)
 
     # see if any of the targets are stochastic and not sampled in the mcmc
-    target_nodes <- lapply(target, get_node)
-    target_node_names <- vapply(target_nodes,
-      member,
-      "unique_name",
-      FUN.VALUE = character(1)
-    )
-    existing_variables <- target_node_names %in% names(mcmc_dag_variables)
-    have_distributions <- vapply(target_nodes,
-      has_distribution,
-      FUN.VALUE = logical(1)
-    )
-    new_stochastics <- have_distributions & !existing_variables
-    if (any(new_stochastics)) {
-      n_stoch <- sum(new_stochastics)
-      msg <- cli::format_error(
-        c(
-          "{.arg nsim} must be set to sample {.cls greta array}s not in MCMC \\
-          samples",
-          "the greta {cli::qty(n_stoch)} arra{?ys/y} \\
-          {.var {names(target)[new_stochastics]}} {cli::qty(n_stoch)} \\
-          {?have distributions and are/has a distribution and is} not in the \\
-          MCMC samples, so cannot be calculated from the samples alone.",
-          "Set {.arg nsim} if you want to sample them conditionally on the \\
-          MCMC samples"
-        )
-      )
-      stop(
-        msg,
-        call. = FALSE
-      )
-    }
-  }
+    check_targets_stochastic_and_not_sampled(target, mcmc_dag_variables)
 
-  dag$define_tf()
+  }
 
   dag$target_nodes <- lapply(target, get_node)
   names(dag$target_nodes) <- names(target)
 
   # if we're doing stochastic sampling, subsample the draws
   if (stochastic) {
+    # TF1/2 check todo
+    # might rename draws to indicate that it is a matrix, for readability
     draws <- as.matrix(draws)
     n_samples <- nrow(draws)
 
@@ -366,22 +345,17 @@ calculate_greta_mcmc_list <- function(target,
     replace <- FALSE
     if (nsim > n_samples) {
       replace <- TRUE
-      msg <- cli::format_warning(
+      cli::cli_warn(
         "{.arg nsim} was greater than the number of posterior samples in \\
         values, so posterior samples had to be drawn with replacement"
       )
-      warning(
-        msg,
-        call. = FALSE
-      )
     }
 
     rows <- sample.int(n_samples, nsim, replace = replace)
     draws <- draws[rows, , drop = FALSE]
 
     # add the batch size to the data list
-    dag$set_tf_data_list("batch_size", as.integer(nsim))
-
+    # assign
     # pass these values in as the free state
     trace <- dag$trace_values(draws,
       trace_batch_size = trace_batch_size,
@@ -393,7 +367,9 @@ calculate_greta_mcmc_list <- function(target,
 
     # for deterministic posterior prediction, just trace the target for each
     # chain
+
     values <- lapply(draws,
+  #double check the trace value part - can pronbanly just do that here
       dag$trace_values,
       trace_batch_size = trace_batch_size
     )
@@ -414,11 +390,12 @@ calculate_greta_mcmc_list <- function(target,
 }
 
 calculate_list <- function(target, values, nsim, tf_float, env) {
-  stochastic <- !is.null(nsim)
-
+  ### browser()
   fixed_greta_arrays <- list()
 
-  if (!identical(values, list())) {
+  values_exist <- !identical(values, list())
+
+  if (values_exist) {
 
     # check the list of values makes sense, and return these and the
     # corresponding greta arrays (looked up by name in environment env)
@@ -428,99 +405,112 @@ calculate_list <- function(target, values, nsim, tf_float, env) {
   }
 
   all_greta_arrays <- c(fixed_greta_arrays, target)
-  # define the dag and TF graph
+
   dag <- dag_class$new(all_greta_arrays, tf_float = tf_float)
 
-  # convert to nodes, and add tensor names to values
-  fixed_nodes <- lapply(fixed_greta_arrays, get_node)
-  names(values) <- vapply(fixed_nodes, dag$tf_name, FUN.VALUE = character(1))
-
-  # change dag mode to sampling
-  dag$mode <- "all_sampling"
-
-  dag$define_tf()
-  tfe <- dag$tf_environment
-
-  # build and send a dict for the fixed values
-  fixed_nodes <- lapply(
-    fixed_greta_arrays,
-    get_node
-  )
-
-  names(values) <- vapply(fixed_nodes,
-    dag$tf_name,
-    FUN.VALUE = ""
-  )
-
-  # check we can do the calculation
+  stochastic <- !is.null(nsim)
+  ### browser()
   if (stochastic) {
 
-    # check there are no variables without distributions (or whose children have
-    # distributions - for lkj & wishart) that aren't given fixed values
-    variables <- dag$node_list[dag$node_types == "variable"]
-    have_distributions <- vapply(variables,
-      has_distribution,
-      FUN.VALUE = logical(1)
-    )
-    any_child_has_distribution <- function(variable) {
-      have_distributions <- vapply(variable$children,
-        has_distribution,
-        FUN.VALUE = logical(1)
-      )
-      any(have_distributions)
-    }
-    children_have_distributions <- vapply(variables,
-      any_child_has_distribution,
-      FUN.VALUE = logical(1)
-    )
+    check_if_unsampleable_and_unfixed(fixed_greta_arrays, dag)
 
-    unsampleable <- !have_distributions & !children_have_distributions
-    fixed_node_names <- vapply(fixed_nodes,
-      member,
-      "unique_name",
-      FUN.VALUE = character(1)
-    )
-    unfixed <- !names(variables) %in% fixed_node_names
-
-    if (any(unsampleable & unfixed)) {
-      msg <- cli::format_error(
-        # NOTE:
-        # is it possible to identify the names of these arrays or variables?
-        "the target {.cls greta_array}s are related to variables that do not \\
-        have distributions so cannot be sampled"
-      )
-      stop(
-        msg,
-        call. = FALSE
-      )
-    }
   } else {
 
     # check there are no unspecified variables on which the target depends
     lapply(target, check_dependencies_satisfied, fixed_greta_arrays, dag, env)
   }
 
+  # TF1/2 check todo
+  # need to wrap this in tf_function I think?
+  if (Sys.getenv("GRETA_DEBUG") == "true") {
+  browser()
+  }
+  values <- calculate_target_tensor_list(
+    dag = dag,
+    fixed_greta_arrays = fixed_greta_arrays,
+    values = values,
+    stochastic = stochastic,
+    target = target,
+    nsim = nsim
+  )
+
+  return(values)
+  # TF1/2 check
+  # could potentially not run this list in the correct way, in that
+  # it might result in running it twice with different seeds, rather than
+  # simultaneously
+  # assign("calculate_target_tensor_list", target_tensor_list, envir = tfe)
+
+#   # add values or data not specified by the user
+#   data_list <- dag$get_tf_data_list()
+#   missing <- !names(data_list) %in% names(values)
+#
+#   # send list to tf environment and roll into a dict
+#
+}
+
+
+calculate_target_tensor_list <- function(
+  dag,
+  fixed_greta_arrays,
+  values,
+  stochastic,
+  target,
+  nsim
+) {
+  # define the dag and TF graph
+  ### browser()
+  # change dag mode to sampling
+  dag$mode <- "all_sampling"
+
+  # convert to nodes, and add tensor names to values
+  fixed_nodes <- lapply(fixed_greta_arrays, get_node)
+
+  value_names <- vapply(fixed_nodes, dag$tf_name, FUN.VALUE = character(1))
+
+  tfe <- dag$tf_environment
+
+  # add the batch size to the data list and the greta stash (for sub-dags)
+  batch_size <- ifelse(stochastic, as.integer(nsim), 1L)
+  assign("batch_size", batch_size, envir = tfe)
+  assign("batch_size", batch_size, envir = greta_stash)
+
+  values <- lapply(values, add_first_dim)
+  values <- lapply(values, tile_first_dim, batch_size)
+
+  mapply(
+    FUN = assign,
+    value_names,
+    values,
+    MoreArgs = list(
+      envir = tfe
+    )
+  )
+
   # look up the tf names of the target greta arrays (under sampling)
   # create an object in the environment that's a list of these, and sample that
   target_nodes <- lapply(target, get_node)
   target_names_list <- lapply(target_nodes, dag$tf_name)
+
+  # this is taking advantage of non-eager mode
+  # TF1/2 check
+  # might need to use some of the tensorflow creation function
+  # approaches (in as_tf_function + generate_log_prob_function)
+  dag$define_tf(target_nodes = target_nodes)
+
+  # browser()
+  # look up the tf names of the target greta arrays (under sampling)
+  # create an object in the environment that's a list of these, and sample that
+  target_nodes <- lapply(target, get_node)
+  target_names_list <- lapply(target_nodes, dag$tf_name)
+
+  ## TF1/2 OK so the error with Wishart and cholesky is happening here
+  ## I feel as thought this isn't the "problem" per se, but it is where the
+  ## matrix of 1s is returned. So seems to me we first expose the problem here
   target_tensor_list <- lapply(target_names_list, get, envir = tfe)
-  assign("calculate_target_tensor_list", target_tensor_list, envir = tfe)
+  ## It looks like it is getting all_sampling_operation_2 and not
+  ## all_sampling_operation_1
+  target_tensor_list_array <- lapply(target_tensor_list, as.array)
 
-  # add the batch size to the data list
-  batch_size <- ifelse(stochastic, as.integer(nsim), 1L)
-  dag$set_tf_data_list("batch_size", batch_size)
-
-  # add values or data not specified by the user
-  data_list <- dag$get_tf_data_list()
-  missing <- !names(data_list) %in% names(values)
-
-  # send list to tf environment and roll into a dict
-  values <- lapply(values, add_first_dim)
-  values <- lapply(values, tile_first_dim, batch_size)
-
-  dag$build_feed_dict(values, data_list = data_list[missing])
-
-  # run the sampling
-  dag$tf_sess_run("calculate_target_tensor_list", as_text = TRUE)
+  return(target_tensor_list_array)
 }
