@@ -12,7 +12,10 @@
 #' @examples
 #' # a symmetric, positive definite square matrix
 #' y <- rWishart(1, 4, diag(3))[, , 1]
+#' y
 #' u <- chol(y)
+#' u
+#' chol2symm(u)
 #' identical(y, chol2symm(u))
 #' identical(chol2symm(u), t(u) %*% u)
 #' \dontrun{
@@ -25,41 +28,20 @@ chol2symm <- function(x) {
 
 #' @export
 chol2symm.default <- function(x) {
-  dim <- dim(x)
-  if (length(dim) != 2 || dim[1] != dim[2]) {
-    msg <- cli::format_error(
-      c(
-        "{.fun chol2symm} must have square symmetric matrix, assumed to be \\
-        upper triangular",
-        "{.code dim(x)} returns: {dim(x)}"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+
+  check_chol2symm_square_symmetric_upper_tri_matrix(x)
 
   t(x) %*% x
 }
 
 #' @export
 chol2symm.greta_array <- function(x) {
+  ## TF1/2
+  ## Does this extra coercion need to be here?
+  ## in order for this to dispatch with S3 it must be a greta array already?
   x <- as.greta_array(x)
-  dim <- dim(x)
-  if (length(dim) != 2 || dim[1] != dim[2]) {
-    msg <- cli::format_error(
-      c(
-        "{.fun chol2symm} must have two-dimensional, square, upper-triangular \\
-        {.cls greta_array}s",
-        "{.code dim(x)} returns: {dim(x)}"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+
+  check_chol2symm_2d_square_upper_tri_greta_array(x)
 
   # sum the elements
   op("chol2symm", x,
