@@ -21,7 +21,7 @@ dag_class <- R6Class(
     initialize = function(target_greta_arrays,
                           tf_float = "float32",
                           compile = FALSE) {
-
+      # browser()
       # build the dag
       self$build_dag(target_greta_arrays)
 
@@ -29,12 +29,46 @@ dag_class <- R6Class(
       self$target_nodes <- lapply(target_greta_arrays, get_node)
 
       # set up the tf environment, with a graph
+      # TF1/2 check
+      # not sure if we need to build the new environment in eager mode?
       self$new_tf_environment()
 
       # store the performance control info
       self$tf_float <- tf_float
       self$compile <- compile
+      self$define_tf_trace_values_batch()
+      self$define_tf_log_prob_function()
     },
+
+    define_tf_trace_values_batch = function(){
+      self$tf_trace_values_batch <- tensorflow::tf_function(
+        f = self$define_trace_values_batch
+      )
+    },
+
+    define_tf_log_prob_function = function(){
+      self$tf_log_prob_function <- tensorflow::tf_function(
+        # TF1/2 check
+        # need to check in on all cases of `tensorflow::tf_function()`
+        # as we are getting lots of warnings about retracting
+        f = self$generate_log_prob_function()
+      )
+    },
+
+    tf_log_prob_function = NULL,
+
+    tf_log_prob_function_adjusted = function(free_state){
+      self$tf_log_prob_function(free_state)$adjusted
+    },
+
+    tf_log_prob_function_unadjusted = function(free_state){
+      self$tf_log_prob_function(free_state)$unadjusted
+    },
+
+    # TF1/2 check
+    # built with TF
+    # Not sure if we need tensorflow environments in TF2, given that
+    # everything will be passed as functions?
     new_tf_environment = function() {
       self$tf_environment <- new.env()
       self$tf_graph <- tf$Graph()
@@ -43,10 +77,13 @@ dag_class <- R6Class(
       self$tf_environment$hybrid_data_list <- list()
     },
 
+    # TF1/2 check remove on_graph
+    # built with TF
+    # Not sure if we need this anyore since this information will be handled
+    # by tf_function?
     # execute an expression on this dag's tensorflow graph, with the correct
     # float type
     on_graph = function(expr) {
-
       # temporarily pass float type info to options, so it can be accessed by
       # nodes on definition, without cluncky explicit passing
       old_float_type <- options()$greta_tf_float
@@ -62,35 +99,12 @@ dag_class <- R6Class(
         greta_batch_size = self$tf_environment$batch_size
       )
 
+      # A tf.Graph can be constructed and used directly without a tf.function,
+      # as was required in TensorFlow 1, but this is deprecated and it is
+      # recommended to use a tf.function instead.
       with(self$tf_graph$as_default(), expr)
     },
 
-    # execute an expression in the tensorflow environment
-    tf_run = function(expr, as_text = FALSE) {
-      tfe <- self$tf_environment
-
-      if (as_text) {
-        tfe$expr <- parse(text = expr)
-      } else {
-        tfe$expr <- substitute(expr)
-      }
-
-      on.exit(rm("expr", envir = tfe))
-
-      self$on_graph(with(tfe, eval(expr)))
-    },
-
-    # sess$run() an expression in the tensorflow environment, with the feed dict
-    tf_sess_run = function(expr, as_text = FALSE) {
-      if (!as_text) {
-        expr <- deparse(substitute(expr))
-      }
-
-      expr <- glue::glue("sess$run({expr}, feed_dict = feed_dict)")
-
-      self$tf_run(expr, as_text = TRUE)
-    },
-
     # return a list of nodes connected to those in the target node list
     build_dag = function(greta_array_list) {
       target_node_list <- lapply(greta_array_list, get_node)
@@ -157,6 +171,7 @@ dag_class <- R6Class(
       # if it's an operation, see if it has a distribution (for lkj and
       # wishart) and get mode based on whether the parent has a free state
       if (node_type == "operation") {
+        # browser()
         parent_name <- node$parents[[1]]$unique_name
         parent_stateless <- parent_name %in% stateless_names
         to_sample <- has_distribution(node) & parent_stateless
@@ -201,9 +216,9 @@ dag_class <- R6Class(
     # how to define the node if we're sampling everything (no free state)
     how_to_define_all_sampling = function(node) {
       switch(node_type(node),
-        data = ifelse(has_distribution(node), "sampling", "forward"),
-        operation = ifelse(has_distribution(node), "sampling", "forward"),
-        "sampling"
+             data = ifelse(has_distribution(node), "sampling", "forward"),
+             operation = ifelse(has_distribution(node), "sampling", "forward"),
+             "sampling"
       )
     },
 
@@ -213,21 +228,38 @@ dag_class <- R6Class(
     how_to_define = function(node) {
       switch(self$mode,
 
-        # if doing inference, everything is push-forward
-        all_forward = "forward",
+             # if doing inference, everything is push-forward
+             all_forward = "forward",
 
-        # sampling from prior most nodes are in sampling mode
-        all_sampling = self$how_to_define_all_sampling(node),
+             # sampling from prior most nodes are in sampling mode
+             all_sampling = self$how_to_define_all_sampling(node),
 
-        # sampling from posterior some nodes defined forward, others sampled
-        hybrid = self$how_to_define_hybrid(node)
+             # sampling from posterior some nodes defined forward, others sampled
+             hybrid = self$how_to_define_hybrid(node)
       )
     },
     define_batch_size = function() {
-      self$tf_run(
-        batch_size <- tf$compat$v1$placeholder(dtype = tf$int32)
-      )
+      # TF1/2 check?
+      # pretty sure `batch_size` just now needs to be the input of a function
+      # I'm not even sure that batch_size needs to be a function, it might
+      # just need to be the input to wherever it is used next?
+
+      ## NOTE: when calling `model` there is no `free_state` in `tf_environment`
+      ## Trying out something where the free state is set if there isn't one?
+
+      if (!exists("batch_size", envir = self$tf_environment)) {
+        with(
+          data = self$tf_environment,
+          batch_size <- tf$shape(self$tf_environment$free_state)[0]
+        )
+      }
+
+      # put this in the greta stash, so it can be accessed by other (sub-)dags
+      # if needed, e.g. when using as_tf_function()
+      assign("batch_size", self$tf_environment$batch_size, envir = greta_stash)
+
     },
+
     define_free_state = function(type = c("variable", "placeholder"),
                                  name = "free_state") {
       type <- match.arg(type)
@@ -238,26 +270,44 @@ dag_class <- R6Class(
 
       if (type == "variable") {
 
+        # TF1/2 check
         # tf$Variable seems to have trouble assigning values, if created with
         # numeric (rather than logical) NAs
         vals <- as.logical(vals)
         vals <- t(as.matrix(vals))
 
-        self$on_graph(free_state <- tf$Variable(
+        free_state <- tf$Variable(
           initial_value = vals,
           dtype = tf_float()
-        ))
+        )
       } else {
         shape <- shape(NULL, length(vals))
+        # TF1/2 check?
+        # this `on_graph` part below is probably not needed/wont work because it
+        # is using placeholder and co?
+        #
+        # defining an empty/unknown thing
+        # so in TF2, we might not need to define a free state, we can
+        # define a function that returns these pieces of information
+        # so we will need to define the function relative to the free state
+        # and will not need to define the object
+        # Can we just turn this into a tensor with some arbitrary value?
         self$on_graph(free_state <- tf$compat$v1$placeholder(
           dtype = tf_float(),
           shape = shape
         ))
+
+        # TF1/2 check
+        # instead?
+        # free_state <- tensorflow::as_tensor(
+        #   dtype = tf_float(),
+        #   shape = shape
+        # )
       }
 
       assign(name,
-        free_state,
-        envir = tfe
+             free_state,
+             envir = tfe
       )
     },
 
@@ -268,10 +318,12 @@ dag_class <- R6Class(
       free_state <- get("free_state", envir = tfe)
 
       params <- self$example_parameters(free = TRUE)
+
       lengths <- lengths(params)
 
       if (length(lengths) > 1) {
-        args <- self$on_graph(tf$split(free_state, lengths, axis = 1L))
+        # args <- self$on_graph(tf$split(free_state, lengths, axis = 1L))
+        args <- tf$split(free_state, lengths, axis = 1L)
       } else {
         args <- list(free_state)
       }
@@ -293,55 +345,34 @@ dag_class <- R6Class(
       }
 
       # define all nodes in the environment and on the graph
-      self$on_graph(
-        lapply(target_nodes, function(x) x$define_tf(self))
-      )
+      ## HERE
+      lapply(target_nodes, function(x){
+        # browser()
+        x$define_tf(self)
+      })
 
       invisible(NULL)
     },
 
-    # use core and compilation options to set up a session in this environment
-    define_tf_session = function() {
-      tfe <- self$tf_environment
-      tfe$n_cores <- self$n_cores
-
-      # nolint start
-      self$tf_run(
-        config <- tf$compat$v1$ConfigProto(
-          inter_op_parallelism_threads = n_cores,
-          intra_op_parallelism_threads = n_cores
-        )
-      )
-
-      if (self$compile) {
-        self$tf_run(
-          py_set_attr(
-            config$graph_options$optimizer_options,
-            "global_jit_level",
-            tf$compat$v1$OptimizerOptions$ON_1
-          )
-        )
-      }
-      # nolint end
-
-      # start a session and initialise all variables
-      self$tf_run(sess <- tf$compat$v1$Session(config = config))
-      self$tf_run(sess$run(tf$compat$v1$global_variables_initializer()))
-    },
-
+    # TF1/2 check?
+    # I think we can probably remove this part of things? However I'm not sure
+    # if the "mode" part is going to be imporatnt here?
     # define tf graph in environment; either for forward-mode computation from a
     # free state variable, or for sampling
     define_tf = function(target_nodes = self$node_list) {
-
       # define the free state variable
-      if (self$mode %in% c("all_forward", "hybrid")) {
-        self$define_free_state("placeholder")
+      # TF1/2 check?
+      # pretty sure define_batch_size needs to be passed as an argument to
+      # whatever is above here...if define_tf even needs to exist?
+      # and I think we can remove define_batch_size since
+      # this should just be passed as an argument later?
+
+      if (self$mode != "all_sampling") {
+        self$define_batch_size()
       }
 
-      # define the body of the graph (depending on the mode) and the session
-      self$define_batch_size()
       self$define_tf_body(target_nodes = target_nodes)
-      self$define_tf_session()
+
     },
 
     # define tensor for overall log density and gradients
@@ -351,28 +382,28 @@ dag_class <- R6Class(
       # get all distribution nodes that have a target
       distribution_nodes <- self$node_list[self$node_types == "distribution"]
       target_nodes <- lapply(distribution_nodes, member, "get_tf_target_node()")
-      has_target <- !vapply(target_nodes, is.null, FUN.VALUE = TRUE)
+      has_target <- !are_null(target_nodes)
       distribution_nodes <- distribution_nodes[has_target]
       target_nodes <- target_nodes[has_target]
 
       # get the densities, evaluated at these targets
       densities <- mapply(self$evaluate_density,
-        distribution_nodes,
-        target_nodes,
-        SIMPLIFY = FALSE
+                          distribution_nodes,
+                          target_nodes,
+                          SIMPLIFY = FALSE
       )
 
       # reduce_sum each of them (skipping the batch dimension)
-      self$on_graph(summed_densities <- lapply(densities, tf_sum, drop = TRUE))
+      summed_densities <- lapply(densities, tf_sum, drop = TRUE)
 
       # sum them together
       names(summed_densities) <- NULL
-      self$on_graph(joint_density <- tf$add_n(summed_densities))
+      joint_density <- tf$add_n(summed_densities)
 
       # assign overall density to environment
       assign("joint_density",
-        joint_density,
-        envir = self$tf_environment
+             joint_density,
+             envir = self$tf_environment
       )
 
       # define adjusted joint density
@@ -387,12 +418,12 @@ dag_class <- R6Class(
       # sometimes returning a scalar tensor)
       names(adj) <- NULL
       adj <- match_batches(adj)
-      self$on_graph(total_adj <- tf$add_n(adj))
+      total_adj <- tf$add_n(adj)
 
       # assign overall density to environment
       assign("joint_density_adj",
-        joint_density + total_adj,
-        envir = self$tf_environment
+             joint_density + total_adj,
+             envir = self$tf_environment
       )
     },
 
@@ -411,8 +442,8 @@ dag_class <- R6Class(
       # execute the distribution constructor functions to return a tfp
       # distribution object
       tfp_distribution <- distrib_constructor(tf_parameter_list, dag = self)
-
-      self$tf_evaluate_density(tfp_distribution,
+      self$tf_evaluate_density(
+        tfp_distribution,
         tf_target,
         truncation = distribution_node$truncation,
         bounds = distribution_node$bounds
@@ -435,6 +466,7 @@ dag_class <- R6Class(
         lower <- truncation[[1]]
         upper <- truncation[[2]]
 
+        ## TODO add explaining variables
         if (all(lower == bounds[1])) {
 
           # if only upper is constrained, just need the cdf at the upper
@@ -447,7 +479,7 @@ dag_class <- R6Class(
 
           # if both are constrained, get the log of the integral between them
           offset <- tf$math$log(tfp_distribution$cdf(fl(upper)) -
-            tfp_distribution$cdf(fl(lower)))
+                                  tfp_distribution$cdf(fl(lower)))
         }
 
         ld <- ld - offset
@@ -462,33 +494,108 @@ dag_class <- R6Class(
       get(self$tf_name(node), envir = self$tf_environment)
     },
 
+    ###<<<
+    hessians = function(
+      free_state,
+      nodes = self$target_nodes,
+      which_objective = c(
+        "adjusted",
+        "unadjusted"
+      )
+    ){
+      which_objective <- match.arg(which_objective)
+
+      ga_names <- names(nodes)
+
+      ## TF1/2 retracing
+      ## This is a location where retracting happens in `opt`
+      hessian_list <- lapply(X = nodes,
+             self$calculate_one_hessian,
+             free_state = free_state,
+             which_objective = which_objective)
+      # assign names and return
+      names(hessian_list) <- ga_names
+      hessian_list
+    },
+
+    calculate_one_hessian = function(
+    node,
+    free_state,
+    which_objective = c(
+      "adjusted",
+      "unadjusted"
+    )
+    ) {
+      which_objective <- match.arg(which_objective)
+
+      # temporarily define a new environment
+      tfe_old <- self$tf_environment
+      on.exit(self$tf_environment <- tfe_old)
+      tfe <- self$tf_environment <- new.env()
+
+      # put the free state in the environment, and build out the tf graph
+      tfe$free_state <- free_state
+
+      # get names and dimensions of target greta arrays
+      ga_dim <- node$dim
+      tf_name <- self$tf_name(node)
+
+      # we now make all of the operations define themselves now
+      with(tf$GradientTape() %as% tape_1, {
+        with(tf$GradientTape() %as% tape_2, {
+          self$define_tf()
+          # define the densities
+          self$define_joint_density()
+
+          xs <- get(tf_name, tfe)
+
+          objectives <- list(
+            adjusted = tfe$joint_density_adj,
+            unadjusted = tfe$joint_density
+          )
+
+          # return either of the densities, or a list of both
+          y <- switch(which_objective,
+                      adjusted = objectives$adjusted,
+                      unadjusted = objectives$unadjusted
+          )
+        })
+        g <- tape_2$gradient(y, xs)
+      })
+      h <- tape_1$jacobian(g, xs)
+
+      # reshape from tensor to R dimensions
+      hessian <- array(h$numpy(), dim = hessian_dims(ga_dim))
+
+      hessian
+
+    },
+
+    ###<<<
+
     # return a function to obtain the model log probability from a tensor for
     # the free state
     generate_log_prob_function = function(which = c(
-                                            "adjusted",
-                                            "unadjusted",
-                                            "both"
-                                          )) {
+      "both",
+      "adjusted",
+      "unadjusted"
+    )) {
       which <- match.arg(which)
 
+      # we can only pass the free_state parameter through
+      # we need some way to lexically scope the
+      # batch size and the data
       function(free_state) {
-
         # temporarily define a new environment
         tfe_old <- self$tf_environment
         on.exit(self$tf_environment <- tfe_old)
         tfe <- self$tf_environment <- new.env()
 
-        # copy the placeholders over here, so they aren't recreated
-        data_names <- self$get_tf_names(types = "data")
-        for (name in data_names) {
-          tfe[[name]] <- tfe_old[[name]]
-        }
-        tfe$batch_size <- tfe_old$batch_size
-
         # put the free state in the environment, and build out the tf graph
         tfe$free_state <- free_state
-        self$define_tf_body()
 
+        # we now make all of the operations define themselves now
+        self$define_tf()
         # define the densities
         self$define_joint_density()
 
@@ -499,9 +606,9 @@ dag_class <- R6Class(
 
         # return either of the densities, or a list of both
         result <- switch(which,
-          adjusted = objectives$adjusted,
-          unadjusted = objectives$unadjusted,
-          both = objectives
+                         adjusted = objectives$adjusted,
+                         unadjusted = objectives$unadjusted,
+                         both = objectives
         )
 
         result
@@ -525,8 +632,8 @@ dag_class <- R6Class(
 
       # remove any of these that don't need a free state here (for calculate())
       stateless_names <- vapply(self$variables_without_free_state,
-        self$tf_name,
-        FUN.VALUE = character(1)
+                                self$tf_name,
+                                FUN.VALUE = character(1)
       )
       keep <- !names(parameters) %in% stateless_names
       parameters <- parameters[keep]
@@ -541,36 +648,10 @@ dag_class <- R6Class(
       data_list_name <- glue::glue("{self$mode}_data_list")
       self$tf_environment[[data_list_name]][[element_name]] <- value
     },
-    build_feed_dict = function(dict_list = list(),
-                               data_list = self$get_tf_data_list()) {
-      tfe <- self$tf_environment
-
-      # put the list in the environment temporarily
-      tfe$dict_list <- c(dict_list, data_list)
-      on.exit(rm("dict_list", envir = tfe))
-
-      # roll into a dict in the tf environment
-      self$tf_run(feed_dict <- do.call(dict, dict_list))
-    },
-    send_parameters = function(parameters) {
-
-      # reshape to a row vector if needed
-      if (is.null(dim(parameters))) {
-        parameters <- array(parameters, dim = c(1, length(parameters)))
-      }
-
-      # create a feed dict in the TF environment
-      parameter_list <- list(free_state = parameters)
-
-      # set the batch size to match parameters
-      self$set_tf_data_list("batch_size", nrow(parameters))
-
-      self$build_feed_dict(parameter_list)
-    },
 
     # get adjusted joint log density across the whole dag
     log_density = function() {
-      res <- cleanly(self$tf_sess_run(joint_density_adj))
+      res <- cleanly(self$tf_environment$joint_density_adj)
 
       if (inherits(res, "error")) {
         res <- NA
@@ -578,64 +659,47 @@ dag_class <- R6Class(
 
       res
     },
-    hessians = function() {
-      tfe <- self$tf_environment
-      nodes <- self$target_nodes
 
-      # get names and dimensions of target greta arrays
-      ga_names <- names(nodes)
-      ga_dims <- lapply(nodes, member, "dim")
-
-      # build the hessian tensors if needed
-      if (!exists("hessian_list", envir = tfe)) {
-        tf_names <- vapply(nodes, self$tf_name, FUN.VALUE = "")
-        y <- tfe$joint_density
-        xs <- lapply(tf_names, get, tfe)
-        names(xs) <- NULL
-        tfe$hessian_list <- self$on_graph(tf$hessians(y, xs))
-      }
-
-      # evaluate at the current free state and assign
-      hessian_list <- self$tf_sess_run(hessian_list)
-
-      # reshape from tensor to R dimensions
-      dims <- lapply(ga_dims, hessian_dims)
-      hessian_list <- mapply(array, hessian_list, dims, SIMPLIFY = FALSE)
-
-      # assign names and return
-      names(hessian_list) <- ga_names
-      hessian_list
+    tf_trace_values_batch = NULL,
+    trace_values_batch = function(free_state_batch){
+      lapply(
+        X = self$tf_trace_values_batch(free_state_batch),
+        FUN = as.array
+      )
     },
-    trace_values_batch = function(free_state_batch) {
+
+    define_trace_values_batch = function(free_state_batch) {
 
       # update the parameters & build the feed dict
-      self$send_parameters(free_state_batch)
-
-      tfe <- self$tf_environment
-
       target_tf_names <- lapply(
         self$target_nodes,
         self$tf_name
       )
 
+      # TF1/2 check
+      # maybe remove onexit stuff?
+      tfe_old <- self$tf_environment
+      on.exit(self$tf_environment <- tfe_old)
+      tfe <- self$tf_environment <- new.env()
+
+      # put the free state in the environment, and build out the tf graph
+      tfe$free_state <- free_state_batch
+
+      # we now make all of the operations define themselves now
+      self$define_tf()
+
       target_tensors <- lapply(target_tf_names,
-        get,
-        envir = tfe
+                               get,
+                               envir = tfe
       )
 
-      # evaluate them in the tensorflow environment
-      trace_list <- tfe$sess$run(target_tensors,
-        feed_dict = tfe$feed_dict
-      )
-
-      trace_list
+      return(target_tensors)
     },
 
     # return the current values of the traced nodes, as a named vector
     trace_values = function(free_state,
                             flatten = TRUE,
                             trace_batch_size = Inf) {
-
       # get the number of samples to trace
       n_samples <- nrow(free_state)
       indices <- seq_len(n_samples)
@@ -705,17 +769,7 @@ dag_class <- R6Class(
         }
       }
 
-      # check we didn't time out
-      if (it == maxit) {
-        msg <- cli::format_error(
-          "could not determine the number of independent models in a \\
-          reasonable amount of time"
-        )
-        stop(
-          msg,
-          call. = FALSE
-        )
-      }
+      check_timeout(it, maxit)
 
       # find the cluster IDs
       n <- nrow(r)
@@ -747,22 +801,12 @@ dag_class <- R6Class(
 
     # try to draw a random sample from a distribution node
     draw_sample = function(distribution_node) {
+      # self$check_sampling_implemented(distribution_node)
       tfp_distribution <- self$get_tfp_distribution(distribution_node)
 
       sample <- tfp_distribution$sample
 
-      if (is.null(sample)) {
-        msg <- cli::format_error(
-          c(
-            "sampling is not yet implemented for \\
-            {.val {distribution_node$distribution_name}} distributions"
-          )
-        )
-        stop(
-          msg,
-          call. = FALSE
-        )
-      }
+      check_sampling_implemented(sample, distribution_node)
 
       truncation <- distribution_node$truncation
 
@@ -778,17 +822,7 @@ dag_class <- R6Class(
 
         cdf <- tfp_distribution$cdf
         quantile <- tfp_distribution$quantile
-
-        if (is.null(cdf) | is.null(quantile)) {
-          msg <- cli::format_error(
-            "sampling is not yet implemented for truncated \\
-            {.val {distribution_node$distribution_name}} distributions"
-          )
-          stop(
-            msg,
-            call. = FALSE
-          )
-        }
+        check_truncation_implemented(tfp_distribution, distribution_node)
 
         # generate a random uniform sample of the correct shape and transform
         # through truncated inverse CDF to get draws on truncated scale
