@@ -101,18 +101,14 @@ NULL
   rm("._dummy_in", envir = pf)
 
   if (any(is.na(dummy_out))) {
-    msg <- cli::format_error(
+    cli::cli_abort(
       "subscript out of bounds"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   node <- get_node(x)
   # if this is a data node, also subset the values and pass on
-  if (inherits(node, "data_node")) {
+  if (is.data_node(node)) {
     values_in <- node$value()
     call_list <- as.list(call)[-1]
     call_list[[1]] <- as.name("._values_in")
@@ -171,14 +167,10 @@ NULL
 
   node <- get_node(x)
 
-  if (inherits(node, "variable_node")) {
-    msg <- cli::format_error(
+  if (is.variable_node(node)) {
+    cli::cli_abort(
       "cannot replace values in a variable {.cls greta_array}"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   # rename value since it is confusing when passed to the op
@@ -205,26 +197,19 @@ NULL
   rm("._dummy_in", envir = pf)
 
   if (any(is.na(dummy_out))) {
-    msg <- cli::format_error(
+    cli::cli_abort(
       "subscript out of bounds"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   index <- as.vector(dummy_out)
 
   if (length(index) != length(replacement)) {
-    if (length(index) %% length(replacement) != 0) {
-      msg <- cli::format_error(
+    replacement_is_not_multiple <- length(index) %% length(replacement) != 0
+    if (replacement_is_not_multiple) {
+      cli::cli_abort(
         "number of items to replace is not a multiple of replacement length"
       )
-      stop(
-        msg,
-        call. = FALSE
-      )
     } else {
       replacement <- rep(replacement, length.out = length(index))
     }
@@ -239,7 +224,8 @@ NULL
   new_value[r_index] <- replacement_value
 
   # if either parent has an unknowns array as a value, coerce this to unknowns
-  if (inherits(x_value, "unknowns") | inherits(replacement_value, "unknowns")) {
+  either_are_unknowns <- is.unknowns(x_value) | is.unknowns(replacement_value)
+  if (either_are_unknowns) {
     new_value <- as.unknowns(new_value)
   }
 
@@ -264,14 +250,11 @@ cbind.greta_array <- function(...) {
 
   dims <- lapply(dots, dim)
   ndims <- lengths(dims)
-  if (!all(ndims == 2)) {
-    msg <- cli::format_error(
+  arrays_are_2d <- all(ndims == 2)
+  if (!arrays_are_2d) {
+    cli::cli_abort(
       "all {.cls greta_array}s must be two-dimensional"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   # dimensions
@@ -279,14 +262,11 @@ cbind.greta_array <- function(...) {
   cols <- vapply(dims, `[`, 2, FUN.VALUE = 1)
 
   # check all the same
-  if (!all(rows == rows[1])) {
-    msg <- cli::format_error(
+  same_number_of_rows <- all(rows == rows[1])
+  if (!same_number_of_rows) {
+    cli::cli_abort(
       "all {.cls greta_array}s must be have the same number of rows"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   # output dimensions
@@ -305,14 +285,11 @@ rbind.greta_array <- function(...) {
 
   dims <- lapply(dots, dim)
   ndims <- lengths(dims)
-  if (!all(ndims == 2)) {
-    msg <- cli::format_error(
+  arrays_2d <- all(ndims == 2)
+  if (!arrays_2d) {
+    cli::cli_abort(
       "all {.cls greta_array}s must be two-dimensional"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   # dimensions
@@ -320,14 +297,11 @@ rbind.greta_array <- function(...) {
   cols <- vapply(dims, `[`, 2, FUN.VALUE = 1)
 
   # check all the same
-  if (!all(cols == cols[1])) {
-    msg <- cli::format_error(
+  array_cols_same <- all(cols == cols[1])
+  if (!array_cols_same) {
+    cli::cli_abort(
       "all {.cls greta_array}s must be have the same number of columns"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   # output dimensions
@@ -363,7 +337,7 @@ abind.default <- function(...,
   # error nicely if they don't have abind installed
   abind_installed <- requireNamespace("abind", quietly = TRUE)
   if (!abind_installed) {
-    msg <- cli::format_error(
+    cli::cli_abort(
       c(
         "{.fun abind} is being called on R arrays (not {.cls greta_array}s), \\
         but the {.pkg abind} package is not installed",
@@ -371,10 +345,6 @@ abind.default <- function(...,
         "{.code install.packages('abind')}"
       )
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   call <- sys.call()
@@ -405,50 +375,45 @@ abind.greta_array <- function(...,
   # nolint end
 
   if (user_set_args) {
-    msg <- cli::format_warning(
+    cli::cli_warn(
       "only the argument {.arg along} is supported when using {.fun abind} \\
       with {.cls greta_array}s, any other arguments will be ignored"
     )
-    warning(
-      msg,
-      call. = FALSE
-    )
   }
 
   arg_list <- list(...)
 
   # drop any NULLs
-  to_discard <- vapply(arg_list, is.null, FUN.VALUE = FALSE)
+  to_discard <- are_null(arg_list)
   if (any(to_discard)) {
     arg_list <- arg_list[!to_discard]
   }
 
   # get N first, in case they used the default value for along
   dims <- lapply(arg_list, dim)
-  n <- max(vapply(dims, length, FUN.VALUE = 1L))
+  n <- max(lengths(dims))
 
   # needed to keep the same formals as abind
   N <- n # nolint
   along <- as.integer(force(along))
 
   # rationalise along, and pad N if we're prepending/appending a dimension
+  ## TODO add explaining variable here
   if (along < 1 || along > n || (along > floor(along) &&
     along < ceiling(along))) {
     n <- n + 1
     along <- max(1, min(n + 1, ceiling(along)))
   }
 
-  if (!(along %in% 0:n)) {
-    msg <- cli::format_error(
+  # TODO revisit checking functions here
+  along_outside_0_n <- !(along %in% 0:n)
+  if (along_outside_0_n) {
+    cli::cli_abort(
       c(
         "{.arg along} must be between 0 and {n}",
         "Instead {.arg along} was {.val {along}}"
       )
     )
-    stop(
-      msg,
-      call. = FALSE
-      )
   }
 
   pre <- seq(from = 1, len = along - 1)
@@ -459,7 +424,7 @@ abind.greta_array <- function(...,
     arg_list,
     function(x) {
       dim <- dim(x)
-      if (length(dim) == n - 1) {
+      if (n_dim(x) == n - 1) {
         dim(x) <- c(dim[pre], 1, dim[post])
       }
       x
@@ -471,18 +436,15 @@ abind.greta_array <- function(...,
   dim_out <- rep(NA, n)
   for (dim in seq_len(n)[-along]) {
     this_dim <- vapply(dims, `[`, dim, FUN.VALUE = 1L)
-    if (!all(this_dim == this_dim[1])) {
-      msg <- cli::format_error(
+    dim_varying <- !all(this_dim == this_dim[1])
+    if (dim_varying) {
+      cli::cli_abort(
         c(
           "all {.cls greta_array}s must have the same dimensions except on \\
           the {.arg along} dimension",
           "However, dimension {dim} had varying sizes: {this_dim}"
         )
       )
-      stop(
-        msg,
-        call. = FALSE
-      )
     } else {
       dim_out[dim] <- this_dim[1]
     }
@@ -508,17 +470,14 @@ c.greta_array <- function(...) {
   args <- list(...)
 
   # drop NULLs from the list
-  is_null <- vapply(args, is.null, FUN.VALUE = FALSE)
+  is_null <- are_null(args)
   args <- args[!is_null]
 
   # try to coerce to greta arrays
   args <- lapply(args, as.greta_array, optional = TRUE)
 
   # return a list if they aren't all greta arrays
-  is_greta_array <- vapply(args,
-    inherits, "greta_array",
-    FUN.VALUE = FALSE
-  )
+  is_greta_array <- are_greta_array(args)
 
   if (!all(is_greta_array)) {
     return(args)
@@ -528,7 +487,7 @@ c.greta_array <- function(...) {
   arrays <- lapply(args, flatten)
 
   # get output dimensions
-  length_vec <- vapply(arrays, length, FUN.VALUE = 1)
+  length_vec <- lengths(arrays)
   dim_out <- c(sum(length_vec), 1L)
 
   # create the op, expanding 'arrays' out to match op()'s dots input
@@ -570,18 +529,13 @@ length.greta_array <- function(x) {
 
   dims <- value
 
-  if (is.null(dims)) {
-    dims <- length(x)
-  }
+  dims <- dims %||% length(x)
 
+  # TODO revisit logic / checking functions here
   if (length(dims) == 0L) {
-    msg <- cli::format_error(
+    cli::cli_abort(
       "length-0 dimension vector is invalid"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   if (length(dims) == 1L) {
@@ -589,51 +543,41 @@ length.greta_array <- function(x) {
   }
 
   if (any(is.na(dims))) {
-    msg <- cli::format_error(
+    cli::cli_abort(
       "the dims contain missing values"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   dims <- as.integer(dims)
 
-  if (any(dims < 0L)) {
-    msg <- cli::format_error(
+  dims_contain_negatives <- any(dims < 0L)
+  if (dims_contain_negatives) {
+    cli::cli_abort(
       c(
         "the dims contain negative values:",
         "{.code dim(x)} returns {dim(x)}"
       )
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   prod_dims <- prod(dims)
   len <- length(x)
-
+  is_scalar <- len == 1
   # if x isn't a scalar and the numbers of elements don't match, error
-  if (len != 1 && prod_dims != len) {
-    msg <- cli::format_error(
-      c(
+  n_elem_not_match <- prod_dims != len
+  if (!is_scalar && n_elem_not_match) {
+    cli::cli_abort(
         "dims [product {prod_dims}] do not match the length of object [{len}]"
-      )
     )
-    stop(
-      msg,
-      call. = FALSE
-      )
   }
 
   # change the values similarly
   new_value <- get_node(x)$value()
   new_value <- array(new_value, dim = dims)
 
-  if (!identical(dim(x), dims) && len == 1) {
+  unmatch_dim <- !identical(dim(x), dims)
+
+  if (unmatch_dim && is_scalar) {
 
     # if the dims don't match, but x is a scalar, expand it to the required
     # dimension
@@ -669,37 +613,10 @@ head.greta_array <- function(x, n = 6L, ...) { # nolint
 
   stopifnot(length(n) == 1L)
 
-  # use default behaviour for R < 4.0.0
-  if (getRversion() < "4.0.0") {
+  ans <- head.matrix(x, n, ...)
 
-    # if x is matrix-like, take the top n rows
-    if (length(dim(x)) == 2) {
-      nrx <- nrow(x)
-      if (n < 0L) {
-        n <- max(nrx + n, 0L)
-      } else {
-        n <- min(n, nrx)
-      }
+  ans
 
-      ans <- x[seq_len(n), , drop = FALSE]
-    } else {
-      # otherwise, take the first n elements
-
-      if (n < 0L) {
-        n <- max(length(x) + n, 0L)
-      } else {
-        n <- min(n, length(x))
-      }
-
-      ans <- x[seq_len(n)]
-    }
-
-  } else if (getRversion() >= "4.0.0") {
-
-    ans <- head.matrix(x, n, ...)
-
-  }
-    ans
 }
 
 #' @export
@@ -709,41 +626,10 @@ tail.greta_array <- function(x, n = 6L, ...) { # nolint
 
   stopifnot(length(n) == 1L)
 
-  # use default behaviour for R < 4.0.0
-  if (getRversion() < "4.0.0") {
+  ans <- tail.matrix(x, n, ...)
 
-    # if x is matrix-like, take the top n rows
-    if (length(dim(x)) == 2) {
-      nrx <- nrow(x)
+  ans
 
-      if (n < 0L) {
-        n <- max(nrx + n, 0L)
-      } else {
-        n <- min(n, nrx)
-      }
-
-      sel <- as.integer(seq.int(to = nrx, length.out = n))
-      ans <- x[sel, , drop = FALSE]
-    } else {
-      # otherwise, take the first n elements
-
-      xlen <- length(x)
-
-      if (n < 0L) {
-        n <- max(xlen + n, 0L)
-      } else {
-        n <- min(n, xlen)
-      }
-
-      ans <- x[seq.int(to = xlen, length.out = n)]
-    }
-
-  } else if (getRversion() >= "4.0.0") {
-
-    ans <- tail.matrix(x, n, ...)
-
-  }
-    ans
 }
 
 #' @rdname overloaded
@@ -763,24 +649,17 @@ diag.greta_array <- function(x = 1, nrow, ncol) {
   dim <- dim(x)
 
   # check the rank isn't too high
-  if (length(dim) != 2) {
-    msg <- cli::format_error(
-      "cannot only extract the diagonal from a node with exactly two \\
+  if (!is_2d(x)) {
+    cli::cli_abort(
+      "Cannot only extract the diagonal from a node with exactly two \\
       dimensions"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
-  if (dim[1] != dim[2]) {
-    msg <- cli::format_error(
-      "diagonal elements can only be extracted from square matrices"
-    )
-    stop(
-      msg,
-      call. = FALSE
+  is_square <- dim[1] != dim[2]
+  if (is_square) {
+    cli::cli_abort(
+      "Diagonal elements can only be extracted from square matrices"
     )
   }
 
