@@ -30,24 +30,7 @@ as.greta_array.logical <- function(x, optional = FALSE, original_x = x, ...) {
 #' @export
 as.greta_array.data.frame <- function(x, optional = FALSE,
                                       original_x = x, ...) {
-  classes <- vapply(x, class, "")
-  valid <- classes %in% c("numeric", "integer", "logical")
-
-  if (!optional & !all(valid)) {
-    invalid_types <- unique(classes[!valid])
-    msg <- cli::format_error(
-      c(
-        "{.cls greta_array} must contain the same type",
-        "Cannot coerce a {.cls data.frame} to a {.cls greta_array} unless \\
-        all columns are {.cls numeric, integer} or {.cls logical}. This \\
-        dataframe had columns of type: {.cls {invalid_types}}"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_greta_data_frame(x, optional)
 
   as.greta_array.numeric(as.matrix(x),
     optional = optional,
@@ -60,25 +43,12 @@ as.greta_array.data.frame <- function(x, optional = FALSE,
 # or numeric
 #' @export
 as.greta_array.matrix <- function(x, optional = FALSE, original_x = x, ...) {
-  if (!is.numeric(x)) {
-    if (is.logical(x)) {
+
+  check_greta_array_type(x, optional)
+
+  if (!is.numeric(x) && is.logical(x)) {
       x[] <- as.numeric(x[])
-    } else if (!optional) {
-      msg <- cli::format_error(
-        c(
-          "{.cls greta_array} must contain the same type",
-          "Cannot coerce {.cls matrix} to a {.cls greta_array} unless it is \\
-          {.cls numeric}, {.cls integer} or {.cls logical}. This \\
-          {.cls matrix} had type:",
-          "{.cls {class(as.vector(x))}}"
-        )
-      )
-      stop(
-        msg,
-        call. = FALSE
-      )
     }
-  }
 
   as.greta_array.numeric(x,
     optional = optional,
@@ -91,24 +61,11 @@ as.greta_array.matrix <- function(x, optional = FALSE, original_x = x, ...) {
 # or numeric
 #' @export
 as.greta_array.array <- function(x, optional = FALSE, original_x = x, ...) {
-  if (!optional & !is.numeric(x)) {
-    if (is.logical(x)) {
+
+  check_greta_array_type(x, optional)
+
+  if (!optional && !is.numeric(x) && is.logical(x)) {
       x[] <- as.numeric(x[])
-    } else {
-      msg <- cli::format_error(
-        c(
-          "{.cls greta_array} must contain the same type",
-          "Cannot coerce {.cls array} to a {.cls greta_array} unless it is \\
-          {.cls numeric}, {.cls integer} or {.cls logical}. This {.cls array} \\
-          had type:",
-          "{.cls {class(as.vector(x))}}"
-        )
-      )
-      stop(
-        msg,
-        call. = FALSE
-      )
-    }
   }
 
   as.greta_array.numeric(x,
@@ -121,17 +78,8 @@ as.greta_array.array <- function(x, optional = FALSE, original_x = x, ...) {
 # finally, reject if there are any missing values, or set up the greta_array
 #' @export
 as.greta_array.numeric <- function(x, optional = FALSE, original_x = x, ...) {
-  if (!optional & any(!is.finite(x))) {
-    msg <- cli::format_error(
-      c(
-        "{.cls greta_array} must not contain missing or infinite values"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_missing_infinite_values(x, optional)
+
   as.greta_array.node(data_node$new(x),
     optional = optional,
     original_x = original_x,
@@ -152,17 +100,13 @@ as.greta_array.node <- function(x, optional = FALSE, original_x = x, ...) {
 #' @export
 as.greta_array.default <- function(x, optional = FALSE, original_x = x, ...) {
   if (!optional) {
-    msg <- cli::format_error(
+    cli::cli_abort(
       c(
         "Object cannot be coerced to {.cls greta_array}",
         "Objects of class {.cls {paste(class(x), collapse = ' or ')}} cannot \\
         be coerced to a {.cls greta_array}"
       )
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   # return x before we started messing with it
@@ -171,16 +115,47 @@ as.greta_array.default <- function(x, optional = FALSE, original_x = x, ...) {
 
 # print method
 #' @export
-print.greta_array <- function(x, ...) {
+print.greta_array <- function(x, ..., n = 10) {
   node <- get_node(x)
-  text <- glue::glue(
-    "greta array ({node$description()})\n\n\n"
-  )
 
-  cat(text)
-  print(node$value(), ...)
+  node_desc <- node$cli_description()
+
+  cli::cli_text("{.pkg greta} array {.cls {node_desc}}")
+  cli::cli_text("\n")
+
+  if (is.unknowns(node$value())){
+    return(print(node$value(), ..., n = n))
+  }
+
+  x_val <- node$value()
+  n_print <- getOption("greta.print_max") %||% n
+
+  n_unknowns <- length(x_val)
+  x_head <- head(x_val, n = n_print)
+  remaining_vals <- n_unknowns - n_print
+
+  # print with question marks
+  print.default(x_head, quote = FALSE, max = n)
+
+  cli::cli_text("\n")
+
+  if (remaining_vals <= 0) {
+    return(invisible(x_val))
+  }
+
+  if (remaining_vals > 0 ) {
+    cli::cli_alert_info(
+      text = c(
+        "i" = "{remaining_vals} more values\n",
+        "i" = "Use {.code print(n = ...)} to see more values"
+      )
+    )
+  }
+
+
 }
 
+
 # summary method
 #' @export
 summary.greta_array <- function(object, ...) {
@@ -226,7 +201,7 @@ print.summary.greta_array <- function(x, ...) {
     distribution_text <- ""
   }
 
-  if (inherits(x$values, "unknowns")) {
+  if (is.unknowns(x$values)) {
     values_text <- "\n  (values currently unknown)"
   } else {
     values_print <- capture.output(summary(x$values))
@@ -267,21 +242,13 @@ get_node <- function(x) {
 
 # check for and get representations
 representation <- function(x, name, error = TRUE) {
-  if (inherits(x, "greta_array")) {
+  if (is.greta_array(x)) {
     x_node <- get_node(x)
   } else {
     x_node <- x
   }
   repr <- x_node$representations[[name]]
-  if (error && is.null(repr)) {
-    msg <- cli::format_error(
-      "{.cls greta_array} has no representation {.var name}"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_has_representation(repr, name, error)
   repr
 }
 
@@ -290,6 +257,23 @@ has_representation <- function(x, name) {
   !is.null(repr)
 }
 
+anti_representation <- function(x, name, error = TRUE) {
+  if (is.greta_array(x)) {
+    x_node <- get_node(x)
+  } else {
+    x_node <- x
+  }
+  repr <- x_node$anti_representations[[name]]
+  check_has_anti_representation(repr, name, error)
+  repr
+}
+
+
+has_anti_representation <- function(x, name){
+  repr <- anti_representation(x, name, error = FALSE)
+  !is.null(repr)
+}
+
 # helper function to make a copy of the greta array & tensor
 copy_representation <- function(x, name) {
   repr <- representation(x, name)
