@@ -1,4 +1,15 @@
-greta_install_miniconda <- function(timeout) {
+#' Installs miniconda
+#'
+#' This installs miniconda using [reticulate::install_miniconda()] inside
+#'   [callr::r_process_options()]. Used internally by [install_greta_deps()].
+#'   We mostly recommend users use [install_greta_deps()] to manage their
+#'   python dependency installation.
+#'
+#' @param timeout time (minutes) until installation stops. Default is 5 minutes.
+#'
+#' @return nothing - installs miniconda.
+#' @export
+greta_install_miniconda <- function(timeout = 5) {
 
   stdout_file <- create_temp_file("out-miniconda")
   stderr_file <- create_temp_file("err-miniconda")
@@ -25,8 +36,4 @@ greta_install_miniconda <- function(timeout) {
   greta_stash$miniconda_notes <- install_miniconda_process$output_notes
   greta_stash$miniconda_error <- install_miniconda_process$output_error
 
-  cli::cli_ul("To see full installation notes run:")
-  cli::cli_ul("{.code greta_notes_install_miniconda_output()}")
-  cli::cli_ul("To see any error messages, run:")
-  cli::cli_ul("{.code greta_notes_install_miniconda_error()}")
 }
