@@ -24,7 +24,7 @@ NULL
 #'   instability during sampling.
 #'
 #' @param compile whether to apply
-#'   [XLA JIT compilation](https://www.tensorflow.org/xla) to
+#'   [XLA JIT compilation](https://openxla.org/xla) to
 #'   the TensorFlow graph representing the model. This may slow down model
 #'   definition, and speed up model evaluation.
 #'
@@ -55,6 +55,9 @@ model <- function(...,
   check_tf_version("error")
 
   # get the floating point precision
+  # TODO
+  # what does it choose as default if both double and single are listed
+  # as default?
   tf_float <- switch(match.arg(precision),
     double = "float64",
     single = "float32"
@@ -64,7 +67,8 @@ model <- function(...,
   target_greta_arrays <- list(...)
 
   # if no arrays were specified, find all of the non-data arrays
-  if (identical(target_greta_arrays, list())) {
+  no_arrays_specified <- identical(target_greta_arrays, list())
+  if (no_arrays_specified) {
     target_greta_arrays <- all_greta_arrays(parent.frame(),
       include_data = FALSE
     )
@@ -79,100 +83,21 @@ model <- function(...,
   target_greta_arrays <- check_greta_arrays(target_greta_arrays, "model")
 
   # get the dag containing the target nodes
+  # TF1/2 check
+  # I don't think we need to use the `compile` flag in TF2 anymore
+  # Well, it will be passed onto the tf_function creation step
   dag <- dag_class$new(target_greta_arrays,
     tf_float = tf_float,
     compile = compile
   )
 
-  # get and check the types
-  types <- dag$node_types
-
   # the user might pass greta arrays with groups of nodes that are unconnected
   # to one another. Need to check there are densities in each graph
-
-  # so find the subgraph to which each node belongs
-  graph_id <- dag$subgraph_membership()
-
-  graphs <- unique(graph_id)
-  n_graphs <- length(graphs)
-
-  # separate messages to avoid the subgraphs issue for beginners
-  if (n_graphs == 1) {
-    density_message <- cli::format_error(
-      c(
-        "none of the {.cls greta_array}s in the model are associated with a \\
-        probability density, so a model cannot be defined"
-        )
-    )
-    variable_message <- cli::format_error(
-      c(
-        "none of the {.cls greta_array}s in the model are unknown, so a model \\
-        cannot be defined"
-        )
-      )
-  } else {
-    density_message <- cli::format_error(
-      c(
-        "the model contains {n_graphs} disjoint graphs",
-        "one or more of these sub-graphs does not contain any \\
-        {.cls greta_array}s that are associated with a probability density, \\
-        so a model cannot be defined"
-      )
-    )
-    variable_message <- cli::format_error(
-        c(
-          "the model contains {n_graphs} disjoint graphs",
-          "one or more of these sub-graphs does not contain any \\
-          {.cls greta_array}s that are unknown, so a model cannot be defined"
-        )
-      )
-  }
-
-  for (graph in graphs) {
-    types_sub <- types[graph_id == graph]
-
-    # check they have a density among them
-    if (!("distribution" %in% types_sub)) {
-      stop(
-        density_message,
-        call. = FALSE
-        )
-    }
-
-    # check they have a variable node among them
-    if (!("variable" %in% types_sub)) {
-      stop(
-        variable_message,
-        call. = FALSE
-        )
-    }
-  }
-
-  # check for unfixed discrete distributions
-  distributions <- dag$node_list[dag$node_types == "distribution"]
-  bad_nodes <- vapply(
-    distributions,
-    function(x) {
-      valid_target <- is.null(x$target) ||
-        inherits(x$target, "data_node")
-      x$discrete && !valid_target
-    },
-    FALSE
-  )
-
-  if (any(bad_nodes)) {
-    msg <- cli::format_error(
-      "model contains a discrete random variable that doesn't have a fixed \\
-      value, so inference cannot be carried out"
-        )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_subgraphs(dag)
+  check_unfixed_discrete_distributions(dag)
 
   # define the TF graph
-  dag$define_tf()
+  # dag$define_tf()
 
   # create the model object and add details
   model <- as.greta_model(dag)
@@ -183,6 +108,11 @@ model <- function(...,
 }
 
 # register generic method to coerce objects to a greta model
+#' @title Convert object to a "greta_model" object
+#' @param x object to convert to greta model
+#' @param ... extra arguments - not used.
+#'
+#' @export
 as.greta_model <- function(x, ...) { # nolint
   UseMethod("as.greta_model", x)
 }
@@ -222,19 +152,7 @@ plot.greta_model <- function(x,
                              y,
                              colour = "#996bc7",
                              ...) {
-  if (!is_DiagrammeR_installed()) {
-    msg <- cli::format_error(
-      c(
-        "the {.pkg DiagrammeR} package must be installed to plot greta models",
-        "install {.pkg DiagrammeR} with:",
-        "{.code install.packages('DiagrammeR')}"
-        )
-      )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_diagrammer_installed()
 
   # set up graph
   dag_mat <- x$dag$adjacency_matrix
@@ -279,11 +197,8 @@ plot.greta_model <- function(x,
 
   # add greta array names where available
   visible_nodes <- lapply(x$visible_greta_arrays, get_node)
-  known_nodes <- vapply(visible_nodes,
-    member,
-    "unique_name",
-    FUN.VALUE = ""
-  )
+  known_nodes <- extract_unique_names(visible_nodes)
+
   known_nodes <- known_nodes[known_nodes %in% names]
   known_idx <- match(known_nodes, names)
   node_labels[known_idx] <- paste(names(known_nodes),
@@ -317,13 +232,7 @@ plot.greta_model <- function(x,
 
   node_names <- lapply(
     parameter_list,
-    function(parameters) {
-      vapply(parameters,
-        member,
-        "unique_name",
-        FUN.VALUE = ""
-      )
-    }
+    extract_unique_names
   )
 
   # for each distribution
@@ -354,7 +263,7 @@ plot.greta_model <- function(x,
     "target"
   )
 
-  keep <- !vapply(targets, is.null, TRUE)
+  keep <- !are_null(targets)
   distrib_idx <- distrib_idx[keep]
 
 
