@@ -48,6 +48,10 @@ greta_stash$numerical_messages <- c(
 #'   when tracing the parameters of interest; reduce this to reduce memory
 #'   demands
 #'
+#' @param compute_options Default is to use CPU only with `cpu_only()`. Use
+#'   `gpu_only()` to use only GPU. In the future we will add more options for
+#'   specifying CPU and GPU use.
+#'
 #' @details For `mcmc()` if `verbose = TRUE`, the progress bar shows
 #'   the number of iterations so far and the expected time to complete the phase
 #'   of model fitting (warmup or sampling). Occasionally, a proposed set of
@@ -104,6 +108,9 @@ greta_stash$numerical_messages <- c(
 #'   argument `trace_batch_size` can be modified to trade-off speed against
 #'   memory usage.
 #'
+#' @note to set a seed with MCMC you must use [tensorflow::set_random_seed()].
+#'   This is due to an internal API with tensorflow. See \url{https://github.com/greta-dev/greta/issues/559} for a thread exploring this.
+#'
 #' @return `mcmc`, `stashed_samples` & `extra_samples` - a
 #'   `greta_mcmc_list` object that can be analysed using functions from the
 #'   coda package. This will contain mcmc samples of the greta arrays used to
@@ -177,107 +184,100 @@ greta_stash$numerical_messages <- c(
 #' o <- opt(m3, hessian = TRUE)
 #' o$hessian
 #' }
-mcmc <- function(model,
-                 sampler = hmc(),
-                 n_samples = 1000,
-                 thin = 1,
-                 warmup = 1000,
-                 chains = 4,
-                 n_cores = NULL,
-                 verbose = TRUE,
-                 pb_update = 50,
-                 one_by_one = FALSE,
-                 initial_values = initials(),
-                 trace_batch_size = 100) {
+mcmc <- function(
+    model,
+    sampler = hmc(),
+    n_samples = 1000,
+    thin = 1,
+    warmup = 1000,
+    chains = 4,
+    n_cores = NULL,
+    verbose = TRUE,
+    pb_update = 50,
+    one_by_one = FALSE,
+    initial_values = initials(),
+    trace_batch_size = 100,
+    compute_options = cpu_only()
+) {
 
-  # check_if_greta_model(model)
-  check_if_greta_array_in_mcmc(model)
+  # set device to be CPU/GPU for the entire run
+  with(tf$device(compute_options), {
 
-  # check the trace batch size
-  trace_batch_size <- check_trace_batch_size(trace_batch_size)
+    # message users about random seeds and GPU usage if they are using GPU
+    message_if_using_gpu(compute_options)
 
-  # find variable names to label samples
-  target_greta_arrays <- model$target_greta_arrays
-  names <- names(target_greta_arrays)
+    check_if_greta_array_in_mcmc(model)
 
-  # check they're not data nodes, provide a useful error message if they are
-  are_data <- vapply(target_greta_arrays,
-    function(x) inherits(get_node(x), "data_node"),
-    FUN.VALUE = FALSE
-  )
+    # check the trace batch size
+    trace_batch_size <- check_trace_batch_size(trace_batch_size)
 
-  if (any(are_data)) {
-    msg <- cli::format_error(
-      c(
-        "data {.cls greta_array}s cannot be sampled",
-        "{.var {names[are_data]}} \\
-        {?is a data/are data} {.cls greta_array}(s)"
-      )
+    check_not_data_greta_arrays(model)
+
+    # get the dag containing the target nodes
+    dag <- model$dag
+
+    chains <- check_positive_integer(chains, "chains")
+
+    # turn initial values into a list if needed (checking the length), and convert
+    # from a named list on the constrained scale to free state vectors
+    initial_values <- prep_initials(initial_values, chains, dag)
+
+    # determine the number of separate samplers to spin up, based on the future
+    # plan
+    max_samplers <- future::nbrOfWorkers()
+
+    # divide chains up between the workers
+    chain_assignment <- sort(rep(
+      seq_len(max_samplers),
+      length.out = chains
+    ))
+
+    # divide the initial values between them
+    initial_values_split <- split(initial_values, chain_assignment)
+
+    n_samplers <- length(initial_values_split)
+
+    # create a sampler object for each parallel job, using these (possibly NULL)
+    # initial values
+    samplers <- lapply(
+      initial_values_split,
+      build_sampler,
+      sampler,
+      model,
+      compute_options = compute_options
     )
-    stop(
-      msg,
-      call. = FALSE)
-  }
 
-  # get the dag containing the target nodes
-  dag <- model$dag
+    # add chain info for printing
+    for (i in seq_len(n_samplers)) {
+      samplers[[i]]$sampler_number <- i
+      samplers[[i]]$n_samplers <- n_samplers
+    }
 
-  chains <- check_positive_integer(chains, "chains")
+    # if verbose = FALSE, make pb_update as big as possible to speed up sampling
+    if (!verbose) {
+      pb_update <- Inf
+    }
 
-  # turn initial values into a list if needed (checking the length), and convert
-  # from a named list on the constrained scale to free state vectors
-  initial_values <- prep_initials(initial_values, chains, dag)
+    # now make it finite
+    pb_update <- min(pb_update, max(warmup, n_samples))
+    pb_update <- max(pb_update, thin + 1)
 
-  # determine the number of separate samplers to spin up, based on the future
-  # plan
-  max_samplers <- future::nbrOfWorkers()
+    run_samplers(
+      samplers = samplers,
+      n_samples = n_samples,
+      thin = thin,
+      warmup = warmup,
+      verbose = verbose,
+      pb_update = pb_update,
+      one_by_one = one_by_one,
+      n_cores = n_cores,
+      from_scratch = TRUE,
+      trace_batch_size = trace_batch_size,
+      compute_options = compute_options
+    )
 
-  # divide chains up between the workers
-  chain_assignment <- sort(rep(seq_len(max_samplers),
-    length.out = chains
-  ))
-
-  # divide the initial values between them
-  initial_values_split <- split(initial_values, chain_assignment)
-
-  n_samplers <- length(initial_values_split)
-
-  # create a sampler object for each parallel job, using these (possibly NULL)
-  # initial values
-  samplers <- lapply(
-    initial_values_split,
-    build_sampler,
-    sampler,
-    model
-  )
-
-  # add chain info for printing
-  for (i in seq_len(n_samplers)) {
-    samplers[[i]]$sampler_number <- i
-    samplers[[i]]$n_samplers <- n_samplers
-  }
-
-  # if verbose = FALSE, make pb_update as big as possible to speed up sampling
-  if (!verbose) {
-    pb_update <- Inf
-  }
-
-  # now make it finite
-  pb_update <- min(pb_update, max(warmup, n_samples))
-  pb_update <- max(pb_update, thin + 1)
-
-  run_samplers(
-    samplers = samplers,
-    n_samples = n_samples,
-    thin = thin,
-    warmup = warmup,
-    verbose = verbose,
-    pb_update = pb_update,
-    one_by_one = one_by_one,
-    n_cores = n_cores,
-    from_scratch = TRUE,
-    trace_batch_size = trace_batch_size
-  )
+    # close `with` setting to control CPU/GPU usage
+  })
 }
 
 #' @importFrom future future resolved value
@@ -290,7 +290,8 @@ run_samplers <- function(samplers,
                          one_by_one,
                          n_cores,
                          from_scratch,
-                         trace_batch_size) {
+                         trace_batch_size,
+                         compute_options) {
 
   # check the future plan is valid, and get information about it
   plan_is <- check_future_plan()
@@ -313,29 +314,25 @@ run_samplers <- function(samplers,
     plan_is$local &
     !is.null(greta_stash$callbacks)
 
+  ## TODO add explaining variable
   if (plan_is$parallel & plan_is$local & length(samplers) > 1) {
-    cores_text <- ifelse(
-      test = n_cores == 1,
-      yes = "1 core",
-      no = glue::glue("up to {n_cores} cores")
-    )
+    cores_text <- compute_text(n_cores, compute_options)
     msg <- glue::glue(
       "\n",
       "running {length(samplers)} samplers in parallel, ",
-      "each on {cores_text}",
+      "{cores_text}",
       "\n\n"
     )
     message(msg)
   }
 
-  if (plan_is$parallel & !plan_is$local) {
+  is_remote_machine <- plan_is$parallel & !plan_is$local
+  if (is_remote_machine) {
 
-    msg <- cli::format_message(
+    cli::cli_inform(
       "running {length(samplers)} \\
       {?sampler on a remote machine/samplers on remote machines}"
-      )
-
-    message(msg)
+    )
 
   }
 
@@ -359,6 +356,7 @@ run_samplers <- function(samplers,
     progress_bar_log_files <- replicate(n_chain, create_log_file(TRUE))
 
     pb_width <- bar_width(n_chain)
+
     for (chain in chains) {
 
       # set the log files
@@ -386,6 +384,7 @@ run_samplers <- function(samplers,
   # dispatch all the jobs
   for (chain in chains) {
     sampler <- samplers[[chain]]
+    sampler$compute_options <- compute_options
     samplers[[chain]] <- dispatch(
       sampler$run_chain(
         n_samples = n_samples,
@@ -467,21 +466,22 @@ stashed_samples <- function() {
     values_draws <- lapply(values_draws, `colnames<-`, trace_names)
 
     # if there are no samples, return a list of NULLs
-    if (nrow(values_draws[[1]]) == 0) {
+    no_samples <- nrow(values_draws[[1]]) == 0
+    if (no_samples) {
       return(replicate(length(samplers), NULL))
     } else {
       thins <- lapply(samplers, member, "thin")
 
       # convert to mcmc objects, passing on thinning
       free_state_draws <- mapply(prepare_draws,
-        draws = free_state_draws,
-        thin = thins,
-        SIMPLIFY = FALSE
+                                 draws = free_state_draws,
+                                 thin = thins,
+                                 SIMPLIFY = FALSE
       )
       values_draws <- mapply(prepare_draws,
-        draws = values_draws,
-        thin = thins,
-        SIMPLIFY = FALSE
+                             draws = values_draws,
+                             thin = thins,
+                             SIMPLIFY = FALSE
       )
 
       # convert to mcmc.list objects
@@ -524,7 +524,8 @@ extra_samples <- function(draws,
                           verbose = TRUE,
                           pb_update = 50,
                           one_by_one = FALSE,
-                          trace_batch_size = 100) {
+                          trace_batch_size = 100,
+                          compute_options = cpu_only()) {
   model_info <- get_model_info(draws)
   samplers <- model_info$samplers
 
@@ -546,7 +547,8 @@ extra_samples <- function(draws,
     one_by_one = one_by_one,
     n_cores = n_cores,
     from_scratch = FALSE,
-    trace_batch_size = trace_batch_size
+    trace_batch_size = trace_batch_size,
+    compute_options = compute_options
   )
 }
 
@@ -554,49 +556,49 @@ extra_samples <- function(draws,
 # 'node'
 #' @importFrom stats qlogis
 to_free <- function(node, data) {
-
   # use reverse mode of bijectors!
 
   lower <- node$lower
   upper <- node$upper
 
+  # TODO
+  # replace these with more informative errors related to the range of values
   unsupported_error <- function() {
-    msg <- cli::format_error(
-        "some provided initial values are outside the range of values their \\
+    cli::cli_abort(
+      "Some provided initial values are outside the range of values their \\
         variables can take"
     )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
   high <- function(x) {
-    if (any(x <= lower)) {
+    initials_outside_support <- any(x <= lower)
+    if (initials_outside_support) {
       unsupported_error()
     }
     log(x - lower)
   }
 
   low <- function(x) {
-    if (any(x >= upper)) {
+    initials_outside_support <- any(x >= upper)
+    if (initials_outside_support) {
       unsupported_error()
     }
     log(upper - x)
   }
 
   both <- function(x) {
-    if (any(x >= upper | x <= lower)) {
+    initials_outside_support <- any(x >= upper | x <= lower)
+    if (initials_outside_support) {
       unsupported_error()
     }
     stats::qlogis((x - lower) / (upper - lower))
   }
 
   fun <- switch(node$constraint,
-    scalar_all_none = identity,
-    scalar_all_high = high,
-    scalar_all_low = low,
-    scalar_all_both = both
+                scalar_all_none = identity,
+                scalar_all_high = high,
+                scalar_all_low = low,
+                scalar_all_both = both
   )
 
   fun(data)
@@ -615,30 +617,16 @@ parse_initial_values <- function(initials, dag) {
 
   # find the elements we have been given initial values for
   tf_names <- vapply(names(initials),
-    function(name, env) {
-      ga <- get(name, envir = env)
-      node <- get_node(ga)
-      dag$tf_name(node)
-    },
-    env = parent.frame(4),
-    FUN.VALUE = ""
+                     function(name, env) {
+                       ga <- get(name, envir = env)
+                       node <- get_node(ga)
+                       dag$tf_name(node)
+                     },
+                     env = parent.frame(4),
+                     FUN.VALUE = ""
   )
 
-  missing_names <- is.na(tf_names)
-  if (any(missing_names)) {
-    bad <- names(tf_names)[missing_names]
-    msg <- cli::format_error(
-      c(
-        "some {.cls greta_array}s passed to {.fun initials} are not associated with \\
-        the model:",
-        "{.var {bad}}"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_greta_arrays_associated_with_model(tf_names)
 
   params <- dag$example_parameters(free = FALSE)
   idx <- match(tf_names, names(params))
@@ -652,33 +640,12 @@ parse_initial_values <- function(initials, dag) {
   # find the corresponding nodes and check they are variable nodes
   forward_names <- glue::glue("all_forward_{dag$node_tf_names}")
   nodes <- dag$node_list[match(tf_names, forward_names)]
-  types <- lapply(nodes, node_type)
-  are_variables <- vapply(types, identical, "variable", FUN.VALUE = FALSE)
 
-  if (!all(are_variables)) {
-    msg <- cli::format_error(
-      "initial values can only be set for variable {.cls greta_array}s"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_nodes_all_variable(nodes)
 
   target_dims <- lapply(params[idx], dim)
   replacement_dims <- lapply(initials, dim)
-  same_dims <- mapply(identical, target_dims, replacement_dims)
-
-  if (!all(same_dims)) {
-    msg <- cli::format_error(
-      "the initial values provided have different dimensions than the named \\
-      {.cls greta_array}s"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_initial_values_correct_dim(target_dims, replacement_dims)
 
   # convert the initial values to their free states
   inits_free <- mapply(to_free, nodes, initials, SIMPLIFY = FALSE)
@@ -698,67 +665,19 @@ prep_initials <- function(initial_values, n_chains, dag) {
 
   # if the user passed a single set of initial values, repeat them for all
   # chains
-  if (inherits(initial_values, "initials")) {
-    is_blank <- identical(initial_values, initials())
-
-    if (!is_blank & n_chains > 1) {
-      msg <- cli::format_message(
-        "only one set of initial values was provided, and was used for \\
-        all chains"
-      )
-      message(msg)
-    }
+  if (is.initials(initial_values)) {
+    inform_if_one_set_of_initials(initial_values, n_chains)
 
     initial_values <- replicate(n_chains,
-      initial_values,
-      simplify = FALSE
-    )
-  } else if (is.list(initial_values)) {
-
-    # if the user provided a list of initial values, check elements and the
-    # length
-    are_initials <- vapply(initial_values, inherits, "initials",
-      FUN.VALUE = FALSE
-    )
-
-    if (all(are_initials)) {
-      n_sets <- length(initial_values)
-
-      if (n_sets != n_chains) {
-        msg <- cli::format_error(
-          c(
-            "the number of provided initial values does not match chains",
-            "{n_sets} set{?s} of initial values were provided, but there\\
-            {cli::qty(n_chains)} {?is only/are} {n_chains}\\
-            {cli::qty(n_chains)} chain{?s}"
-            )
-          )
-        stop(
-          msg,
-          call. = FALSE
-        )
-      }
-    } else {
-      initial_values <- NULL
-    }
-  } else {
-    initial_values <- NULL
-  }
-
-  # error on a bad object
-  if (is.null(initial_values)) {
-    msg <- cli::format_error(
-      c(
-        "{.arg initial_values} must be an initials object created with \\
-        {.fun initials}, or a simple list of initials objects"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
+                                initial_values,
+                                simplify = FALSE
     )
   }
 
+  # TODO: revisit logic here for errors and messages
+  check_initial_values_match_chains(initial_values, n_chains)
+  check_initial_values_correct_class(initial_values)
+
   # convert them to free state vectors
   initial_values <- lapply(
     initial_values,
@@ -779,29 +698,12 @@ initials <- function(...) {
   values <- list(...)
   names <- names(values)
 
-  if (length(names) != length(values)) {
-    msg <- cli::format_error(
-      "all initial values must be named"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_initials_are_named(values)
 
   # coerce to greta-array-like shape
   values <- lapply(values, as_2d_array)
 
-  are_numeric <- vapply(values, is.numeric, FUN.VALUE = FALSE)
-  if (!all(are_numeric)) {
-    msg <- cli::format_error(
-      "initial values must be numeric"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_initials_are_numeric(values)
 
   class(values) <- c("initials", class(values))
   values
@@ -809,7 +711,8 @@ initials <- function(...) {
 
 #' @export
 print.initials <- function(x, ...) {
-  if (identical(x, initials())) {
+  empty_initials <- identical(x, initials())
+  if (empty_initials) {
     cat("an empty greta initials object")
   } else {
     cat("a greta initials object with values:\n\n")
@@ -833,6 +736,9 @@ print.initials <- function(x, ...) {
 #'   estimates, or `TRUE` for maximum *a posteriori* estimates.
 #' @param hessian whether to return a list of *analytically* differentiated
 #'   Hessian arrays for the parameters
+#' @param compute_options Default is to use CPU only with `cpu_only()`. Use
+#'   `gpu_only()` to use only GPU. In the future we will add more options for
+#'   specifying CPU and GPU use.
 #'
 #' @details Because `opt()` acts on a list of greta arrays with possibly
 #'   varying dimension, the `par` and `hessian` objects returned by
@@ -865,40 +771,57 @@ opt <- function(model,
                 tolerance = 1e-6,
                 initial_values = initials(),
                 adjust = TRUE,
-                hessian = FALSE) {
+                hessian = FALSE,
+                compute_options = cpu_only()) {
 
-  # check initial values. Can up the number of chains in the future to handle
-  # random restarts
-  initial_values_list <- prep_initials(initial_values, 1, model$dag)
+  # set device to be CPU/GPU for the entire run
+  with(tf$device(compute_options), {
 
-  # create R6 object of the right type
-  object <- optimiser$class$new(
-    initial_values = initial_values_list[1],
-    model = model,
-    name = optimiser$name,
-    method = optimiser$method,
-    parameters = optimiser$parameters,
-    other_args = optimiser$other_args,
-    max_iterations = max_iterations,
-    tolerance = tolerance,
-    adjust = adjust
-  )
+    # message users about random seeds and GPU usage if they are using GPU
+    message_if_using_gpu(compute_options)
 
-  # run it and get the outputs
-  object$run()
-  outputs <- object$return_outputs()
+    # check initial values. Can up the number of chains in the future to handle
+    # random restarts
+    initial_values_list <- prep_initials(initial_values, 1, model$dag)
 
-  # optionally evaluate the hessians at these parameters (return as hessian for
-  # objective function)
-  if (hessian) {
-    hessians <- model$dag$hessians()
-    hessians <- lapply(hessians, `*`, -1)
-    outputs$hessian <- hessians
-  }
+    # create R6 object of the right type
+    object <- optimiser$class$new(
+      initial_values = initial_values_list[1],
+      model = model,
+      name = optimiser$name,
+      method = optimiser$method,
+      parameters = optimiser$parameters,
+      other_args = optimiser$other_args,
+      max_iterations = max_iterations,
+      tolerance = tolerance,
+      adjust = adjust
+    )
 
-  outputs
+    # run it and get the outputs
+    object$run()
+    outputs <- object$return_outputs()
+
+    # optionally evaluate the hessians at these parameters (return as hessian for
+    # objective function)
+    if (hessian) {
+      which_objective <- ifelse(adjust, "adjusted", "unadjusted")
+      hessians <- model$dag$hessians(
+        # coercing to Variable as TFP free state is a Tensor not a variable
+        # and you can overload a Variable as a Variable without consequence
+        # (hopefully)
+        free_state = tf$Variable(object$free_state),
+        which_objective = which_objective
+      )
+      hessians <- lapply(hessians, `*`, -1)
+      outputs$hessian <- hessians
+    }
+
+    outputs
+
+    # close setting of CPU/GPU usage
+  })
 }
 
 inference_module <- module(dag_class,
-  progress_bar = progress_bar_module
+                           progress_bar = progress_bar_module
 )
