@@ -6,6 +6,9 @@ inference <- R6Class(
   public = list(
     model = NULL,
 
+    # compute information
+    compute_options = NULL,
+
     # RNG seed
     seed = 1,
 
@@ -32,18 +35,14 @@ inference <- R6Class(
                           parameters = list(),
                           seed = get_seed()) {
 
-      # flush the environment and redefine the tensorflow graph if needed
-      if (is.null(model$dag$tf_graph$unique_name)) {
-        model$dag$new_tf_environment()
-        model$dag$define_tf()
-      }
-
       self$parameters <- parameters
       self$model <- model
       free_parameters <- model$dag$example_parameters(free = TRUE)
       free_parameters <- unlist_tf(free_parameters)
       self$n_free <- length(free_parameters)
       self$set_initial_values(initial_values)
+      # maybe inefficient-  potentially speed up?
+
       self$n_traced <- length(model$dag$trace_values(self$free_state))
       self$seed <- seed
     },
@@ -54,14 +53,14 @@ inference <- R6Class(
       if (file.exists(self$trace_log_file)) {
         # Append
         write.table(last_burst_values, self$trace_log_file,
-          append = TRUE,
-          row.names = FALSE, col.names = FALSE
+                    append = TRUE,
+                    row.names = FALSE, col.names = FALSE
         )
       } else {
         # Create file
         write.table(last_burst_values, self$trace_log_file,
-          append = FALSE,
-          row.names = FALSE, col.names = TRUE
+                    append = FALSE,
+                    row.names = FALSE, col.names = TRUE
         )
       }
     },
@@ -86,7 +85,8 @@ inference <- R6Class(
 
     # check and try to autofill a single set of initial values (single vector on
     # free state scale)
-    check_initial_values = function(inits) {
+    check_initial_values = function(inits,
+                                    call = rlang::caller_env()) {
       undefined <- is.na(inits)
 
       # try to fill in any that weren't specified
@@ -103,43 +103,45 @@ inference <- R6Class(
           attempts <- attempts + 1
         }
 
-        if (!valid) {
-          msg <- cli::format_error(
-            c(
-              "Could not find reasonable starting values after \\
-              {attempts} attempts.",
-              "Please specify initial values manually via the \\
-              {.arg initial_values} argument"
-            )
-          )
-          stop(
-            msg,
-            call. = FALSE
-          )
-        }
+        self$check_reasonable_starting_values(valid, attempts)
+
       } else {
 
         # if they were all provided, check they can be be used
         valid <- self$valid_parameters(inits)
-        if (!valid) {
-          msg <- cli::format_error(
-            c(
-              "The log density could not be evaluated at these initial values",
-              "Try using these initials as the values argument in \\
-              {.fun calculate} to see what values of subsequent \\
-              {.cls greta_array}s these initial values lead to."
-            )
-          )
-          stop(
-            msg,
-            call. = FALSE
-          )
-        }
+        self$check_valid_parameters(valid)
+
       }
 
       inits
     },
 
+    check_reasonable_starting_values = function(valid, attempts){
+      if (!valid) {
+        cli::cli_abort(
+          message = c(
+            "Could not find reasonable starting values after \\
+              {attempts} attempts.",
+            "Please specify initial values manually via the \\
+              {.arg initial_values} argument"
+          )
+        )
+      }
+    },
+
+    check_valid_parameters = function(valid){
+      if (!valid) {
+        cli::cli_abort(
+          c(
+            "The log density could not be evaluated at these initial values",
+            "Try using these initials as the {.arg values} argument in \\
+              {.fun calculate} to see what values of subsequent \\
+              {.cls greta_array}s these initial values lead to."
+          )
+        )
+      }
+    },
+
     # check and set a list of initial values
     set_initial_values = function(init_list) {
 
@@ -156,27 +158,23 @@ inference <- R6Class(
     # check whether the model can be evaluated at these parameters
     valid_parameters = function(parameters) {
       dag <- self$model$dag
-      tfe <- dag$tf_environment
-
-      if (!live_pointer("joint_density_adj", envir = tfe)) {
-        dag$on_graph(dag$define_joint_density())
-      }
-
-      dag$send_parameters(parameters)
-      ld <- self$model$dag$log_density()
-      is.finite(ld)
+      tf_parameters <- fl(array(
+        data = parameters,
+        dim = c(1, length(parameters))
+      ))
+      ld <- lapply(
+        dag$tf_log_prob_function(tf_parameters),
+        as.numeric
+      )
+      is.finite(ld$adjusted) && is.finite(ld$unadjusted)
     },
 
     # run a burst of sampling, and put the resulting free state values in
     # last_burst_free_states
     run_burst = function() {
-      msg <- cli::format_error(
+      cli::cli_abort(
         "no method to run a burst in the base inference class"
       )
-      stop(
-        msg,
-        call. = FALSE
-      )
       self$last_burst_free_states <- free_states
     },
 
@@ -187,9 +185,9 @@ inference <- R6Class(
 
         # append the free state trace for each chain
         self$traced_free_state <- mapply(rbind,
-          self$traced_free_state,
-          self$last_burst_free_states,
-          SIMPLIFY = FALSE
+                                         self$traced_free_state,
+                                         self$last_burst_free_states,
+                                         SIMPLIFY = FALSE
         )
       }
 
@@ -200,9 +198,9 @@ inference <- R6Class(
           self$write_trace_to_log_file(last_burst_values)
         }
         self$traced_values <- mapply(rbind,
-          self$traced_values,
-          last_burst_values,
-          SIMPLIFY = FALSE
+                                     self$traced_values,
+                                     last_burst_values,
+                                     SIMPLIFY = FALSE
         )
       }
     },
@@ -278,8 +276,8 @@ sampler <- R6Class(
     initialize = function(initial_values,
                           model,
                           parameters = list(),
-                          seed) {
-
+                          seed,
+                          compute_options) {
       # initialize the inference method
       super$initialize(
         initial_values = initial_values,
@@ -291,16 +289,48 @@ sampler <- R6Class(
       self$n_chains <- nrow(self$free_state)
 
       # duplicate diag_sd if needed
+      ## TODO improve explaining variable here - why does this need to happen?
       n_diag <- length(self$parameters$diag_sd)
       n_parameters <- self$n_free
-      if (n_diag != n_parameters && n_parameters > 1) {
+      multiple_parameters <- n_parameters > 1
+      if (n_diag != n_parameters && multiple_parameters) {
         diag_sd <- rep(self$parameters$diag_sd[1], n_parameters)
         self$parameters$diag_sd <- diag_sd
       }
 
       # define the draws tensor on the tf graph
-      self$define_tf_draws()
+      # define_tf_draws is now used in place of of run_burst
+      self$define_tf_evaluate_sample_batch()
+
     },
+
+    define_tf_evaluate_sample_batch = function(){
+      self$tf_evaluate_sample_batch <- tensorflow::tf_function(
+        f = self$define_tf_draws,
+        input_signature = list(
+          # free state
+          tf$TensorSpec(shape = list(NULL, self$n_free),
+                        dtype = tf_float()),
+          # sampler_burst_length
+          tf$TensorSpec(shape = list(),
+                        dtype = tf$int32),
+          # sampler_thin
+          tf$TensorSpec(shape = list(),
+                        dtype = tf$int32),
+          # sampler_param_vec
+          tf$TensorSpec(shape = list(
+            length(
+              unlist(
+                self$sampler_parameter_values()
+              )
+            )
+          ),
+          dtype = tf_float()
+          )
+        )
+      )
+    },
+
     run_chain = function(n_samples, thin, warmup,
                          verbose, pb_update,
                          one_by_one, plan_is, n_cores, float_type,
@@ -315,32 +345,26 @@ sampler <- R6Class(
       if (!plan_is$parallel & verbose) {
         self$print_sampler_number()
       }
-
       if (plan_is$parallel) {
 
-        # flush the environment
-        dag$new_tf_environment()
+        dag$define_tf_trace_values_batch()
 
-        # set the batch size for multiple chains
-        dag$set_tf_data_list("batch_size", self$n_chains)
+        dag$define_tf_log_prob_function()
 
-        # rebuild the TF graph
-        dag$define_tf()
+        self$define_tf_evaluate_sample_batch()
 
-        # rebuild the TF draws tensor
-        self$define_tf_draws()
       }
 
       # create these objects if needed
       if (from_scratch) {
         self$traced_free_state <- replicate(self$n_chains,
-          matrix(NA, 0, self$n_free),
-          simplify = FALSE
+                                            matrix(NA, 0, self$n_free),
+                                            simplify = FALSE
         )
 
         self$traced_values <- replicate(self$n_chains,
-          matrix(NA, 0, self$n_traced),
-          simplify = FALSE
+                                        matrix(NA, 0, self$n_traced),
+                                        simplify = FALSE
         )
       }
 
@@ -364,14 +388,25 @@ sampler <- R6Class(
 
         # split up warmup iterations into bursts of sampling
         burst_lengths <- self$burst_lengths(warmup,
-          ideal_burst_size,
-          warmup = TRUE
+                                            ideal_burst_size,
+                                            warmup = TRUE
         )
         completed_iterations <- cumsum(burst_lengths)
 
+        # relay between R and tensorflow in a burst to be cpu efficient
         for (burst in seq_along(burst_lengths)) {
-          self$run_burst(burst_lengths[burst])
+          # TF1/2 check todo?
+          # replace with define_tf_draws
+
+          self$run_burst(n_samples = burst_lengths[burst])
+          # align the free state back to the parameters we are tracing
+          # TF1/2 check todo?
+          # this is the tuning stage, might not need to evaluate
+          # / record the parameter values, as they will be thrown away
+          # after warmup - so could remove trace here.
+
           self$trace()
+          # a memory efficient way to calculate summary stats of samples
           self$update_welford()
           self$tune(completed_iterations[burst], warmup)
 
@@ -379,23 +414,23 @@ sampler <- R6Class(
 
             # update the progress bar/percentage log
             iterate_progress_bar(pb_warmup,
-              it = completed_iterations[burst],
-              rejects = self$numerical_rejections,
-              chains = self$n_chains,
-              file = self$pb_file
+                                 it = completed_iterations[burst],
+                                 rejects = self$numerical_rejections,
+                                 chains = self$n_chains,
+                                 file = self$pb_file
             )
 
             self$write_percentage_log(warmup,
-              completed_iterations[burst],
-              stage = "warmup"
+                                      completed_iterations[burst],
+                                      stage = "warmup"
             )
           }
         }
 
         # scrub the free state trace and numerical rejections
         self$traced_free_state <- replicate(self$n_chains,
-          matrix(NA, 0, self$n_free),
-          simplify = FALSE
+                                            matrix(NA, 0, self$n_free),
+                                            simplify = FALSE
         )
         self$numerical_rejections <- 0
       }
@@ -425,22 +460,28 @@ sampler <- R6Class(
         completed_iterations <- cumsum(burst_lengths)
 
         for (burst in seq_along(burst_lengths)) {
-          self$run_burst(burst_lengths[burst], thin = thin)
+          # so these bursts are R objects being passed through to python
+          # and how often to return them
+          # TF1/2 check todo
+          # replace with define_tf_draws
+          self$run_burst(n_samples = burst_lengths[burst],
+                         thin = thin)
+          # trace is it receiving the python
           self$trace()
 
           if (verbose) {
 
             # update the progress bar/percentage log
             iterate_progress_bar(pb_sampling,
-              it = completed_iterations[burst],
-              rejects = self$numerical_rejections,
-              chains = self$n_chains,
-              file = self$pb_file
+                                 it = completed_iterations[burst],
+                                 rejects = self$numerical_rejections,
+                                 chains = self$n_chains,
+                                 file = self$pb_file
             )
 
             self$write_percentage_log(n_samples,
-              completed_iterations[burst],
-              stage = "sampling"
+                                      completed_iterations[burst],
+                                      stage = "sampling"
             )
           }
         }
@@ -487,8 +528,8 @@ sampler <- R6Class(
     # chain dimension
     trace_values = function(trace_batch_size) {
       self$traced_values <- lapply(self$traced_free_state,
-        self$model$dag$trace_values,
-        trace_batch_size = trace_batch_size
+                                   self$model$dag$trace_values,
+                                   trace_batch_size = trace_batch_size
       )
     },
 
@@ -504,15 +545,12 @@ sampler <- R6Class(
 
       if (self$n_chains > 1) {
         n_cores <- self$model$dag$n_cores
+        compute_options <- self$compute_options
 
-        cores_text <- ifelse(
-          test = n_cores == 1,
-          yes = "1 core",
-          no = glue::glue("up to {n_cores} cores")
-        )
+        cores_text <- compute_text(n_cores, compute_options)
 
         msg <- glue::glue(
-          "\n\nrunning {self$n_chains} chains simultaneously on {cores_text}"
+          "\n\nrunning {self$n_chains} chains simultaneously {cores_text}"
         )
       }
 
@@ -620,71 +658,104 @@ sampler <- R6Class(
         }
       }
     },
-    define_tf_draws = function() {
+    # TF1/2 check todo
+    # need to convert this into a TF function
+    define_tf_draws = function(free_state,
+                               sampler_burst_length,
+                               sampler_thin,
+                               sampler_param_vec
+                               # pass values through
+    ) {
+
       dag <- self$model$dag
       tfe <- dag$tf_environment
 
+      # TF1/2 check seed
+      # how do TF2 and TFP use seeds?
       self$set_tf_seed()
 
-      self$define_tf_kernel()
+      sampler_kernel <- self$define_tf_kernel(
+        sampler_param_vec
+      )
 
-      # and the sampler info
-      dag$tf_run(
-        sampler_burst_length <- tf$compat$v1$placeholder(dtype = tf$int32)
-      )
-      dag$tf_run(
-        sampler_thin <- tf$compat$v1$placeholder(dtype = tf$int32)
-      )
+      # TF1/2 check
+      # some sampler parameter values need to be re-run at each iteration to
+      # decide, e.g., the leap step in HMC, which is run inside define_tf_kernel
+      # currently we run `sample_parameter_values` which will randomly pick
+      # an "l" step.
+      # Need to understand if/how tf_function will re-run those values - might
+      # need to pass these arguments directly
 
       # define the whole draws tensor
-      dag$tf_run(
-        sampler_batch <- tfp$mcmc$sample_chain(
-          num_results = tf$math$floordiv(sampler_burst_length, sampler_thin),
-          current_state = free_state,
-          kernel = sampler_kernel,
-          trace_fn = function(current_state, kernel_results) {
-            kernel_results
-          },
-          num_burnin_steps = tf$constant(0L, dtype = tf$int32),
-          num_steps_between_results = sampler_thin,
-          parallel_iterations = 1L
-        )
+      # TF1/2 check
+      # `seed` arg now gets passed to `sample_chain`.
+      # Need to work out how to get sampler_batch() to run as a TF function.
+      # To do that we need to work out how to get the free state
+
+      sampler_batch <- tfp$mcmc$sample_chain(
+        num_results = tf$math$floordiv(sampler_burst_length, sampler_thin),
+        current_state = free_state,
+        kernel = sampler_kernel,
+        trace_fn = function(current_state, kernel_results) {
+          kernel_results
+        },
+        num_burnin_steps = tf$constant(0L, dtype = tf$int32),
+        num_steps_between_results = sampler_thin,
+        parallel_iterations = 1L
+      )
+      return(
+        sampler_batch
       )
     },
 
     # run a burst of the sampler
-    run_burst = function(n_samples, thin = 1L) {
+    # TF1/2 check
+    # this will be removed in favour of the tf_function decorated
+    # define_tf_draws() function that takes in argument values
+    # sampler_burst_length and sampler_thin
+    run_burst = function(n_samples,
+                         thin = 1L) {
       dag <- self$model$dag
       tfe <- dag$tf_environment
 
+      param_vec <- unlist(self$sampler_parameter_values())
       # combine the sampler information with information on the sampler's tuning
       # parameters, and make into a dict
-      sampler_values <- list(
-        free_state = self$free_state,
-        sampler_burst_length = as.integer(n_samples),
-        sampler_thin = as.integer(thin)
-      )
 
-      sampler_dict_list <- c(
-        sampler_values,
-        self$sampler_parameter_values()
-      )
+      # sampler_values <- list(
+      #   # TF1/2 check
+      # do we need free state here anymore?
+      #   free_state = self$free_state,
+      #   sampler_burst_length = as.integer(n_samples),
+      #   sampler_thin = as.integer(thin)
+      # )
+      # # create a function that takes in these arguments ^^
+      # # and then run the code for the sampler_batch
+      #
 
       dag$set_tf_data_list("batch_size", nrow(self$free_state))
-      dag$build_feed_dict(sampler_dict_list)
 
       # run the sampler, handling numerical errors
-      batch_results <- self$sample_carefully(n_samples)
+      batch_results <- self$sample_carefully(
+        free_state = self$free_state,
+        sampler_burst_length = as.integer(n_samples),
+        sampler_thin = as.integer(thin),
+        sampler_param_vec = param_vec
+      )
 
       # get trace of free state and drop the null dimension
-      free_state_draws <- batch_results$all_states
+      if (is.null(batch_results$all_states)){
+        browser()
+      }
+      free_state_draws <- as.array(batch_results$all_states)
 
       # if there is one sample at a time, and it's rejected, conversion from
       # python back to R can drop a dimension, so handle that here. Ugh.
-      if (length(dim(free_state_draws)) != 3) {
+      if (n_dim(free_state_draws) != 3) {
         dim(free_state_draws) <- c(1, dim(free_state_draws))
       }
 
+
       self$last_burst_free_states <- split_chains(free_state_draws)
 
       n_draws <- nrow(free_state_draws)
@@ -697,8 +768,8 @@ sampler <- R6Class(
       if (self$uses_metropolis) {
 
         # log acceptance probability
-        log_accept_stats <- batch_results$trace$log_accept_ratio
-        is_accepted <- batch_results$trace$is_accepted
+        log_accept_stats <- as.array(batch_results$trace$log_accept_ratio)
+        is_accepted <- as.array(batch_results$trace$is_accepted)
         self$accept_history <- rbind(self$accept_history, is_accepted)
         accept_stats_batch <- pmin(1, exp(log_accept_stats))
         self$mean_accept_stat <- mean(accept_stats_batch, na.rm = TRUE)
@@ -708,17 +779,45 @@ sampler <- R6Class(
         self$numerical_rejections <- self$numerical_rejections + bad
       }
     },
-    sample_carefully = function(n_samples) {
+
+    tf_evaluate_sample_batch = NULL,
+
+    sample_carefully = function(free_state,
+                                sampler_burst_length,
+                                sampler_thin,
+                                sampler_param_vec) {
 
       # tryCatch handling for numerical errors
       dag <- self$model$dag
       tfe <- dag$tf_environment
+      # legacy: previously we used `n_samples` not `sampler_burst_length`
+      n_samples <- sampler_burst_length
 
-      # don't use dag$tf_sess_run, to avoid the overhead on parsing expressions
-      result <- cleanly(tfe$sess$run(tfe$sampler_batch,
-        feed_dict = tfe$feed_dict
-      ))
+      result <- cleanly(
+        self$tf_evaluate_sample_batch(
+          free_state = tensorflow::as_tensor(
+            free_state,
+            dtype = tf_float()
+          ),
+          sampler_burst_length = tensorflow::as_tensor(sampler_burst_length),
+          sampler_thin = tensorflow::as_tensor(sampler_thin),
+          sampler_param_vec = tensorflow::as_tensor(
+            sampler_param_vec,
+            dtype = tf_float(),
+            shape = length(sampler_param_vec)
+          )
+        )
+      ) # closing cleanly
 
+      # if it's fine, batch_results is the output
+      # if it's a non-numerical error, it will error
+      # if it's a numerical error, batch_results will be an error object
+      self$check_for_free_state_error(result, n_samples)
+
+      result
+    },
+
+    check_for_free_state_error = function(result, n_samples){
       # if it's fine, batch_results is the output
       # if it's a non-numerical error, it will error
       # if it's a numerical error, batch_results will be an error object
@@ -741,8 +840,8 @@ sampler <- R6Class(
           # otherwise, *one* of these multiple samples was bad. The sampler
           # won't be valid if we just restart, so we need to error here,
           # informing the user how to run one sample at a time
-          msg <- cli::format_error(
-            c(
+          cli::cli_abort(
+            message = c(
               "TensorFlow hit a numerical problem that caused it to error",
               "{.pkg greta} can handle these as bad proposals if you rerun \\
               {.fun mcmc} with the argument {.code one_by_one = TRUE}.",
@@ -752,15 +851,10 @@ sampler <- R6Class(
             )
           )
 
-          stop(
-            msg,
-            call. = FALSE
-          )
         }
       }
-
-      result
     },
+
     sampler_parameter_values = function() {
 
       # random number of integration steps
@@ -780,46 +874,43 @@ hmc_sampler <- R6Class(
       diag_sd = 1
     ),
     accept_target = 0.651,
-    define_tf_kernel = function() {
+
+    define_tf_kernel = function(sampler_param_vec) {
+
       dag <- self$model$dag
       tfe <- dag$tf_environment
 
-      # tensors for sampler parameters
-      dag$tf_run(
-        hmc_epsilon <- tf$compat$v1$placeholder(dtype = tf_float())
-      )
-      dag$tf_run(
-        hmc_l <- tf$compat$v1$placeholder(dtype = tf$int64)
-      )
+      free_state_size <- length(sampler_param_vec) - 2
 
-      # need to pass in the value for this placeholder as a matrix (shape(n, 1))
-      dag$tf_run(
-        hmc_diag_sd <- tf$compat$v1$placeholder(
-          dtype = tf_float(),
-          shape = shape(dim(free_state)[[2]], 1)
-        )
-      )
+      # TF1/2 check
+      # this will likely get replaced...
 
-      # but it step_sizes must be a vector (shape(n, )), so reshape it
-      dag$tf_run(
-        hmc_step_sizes <- tf$reshape(
+      hmc_l <- sampler_param_vec[0]
+      hmc_epsilon <- sampler_param_vec[1]
+      hmc_diag_sd <- sampler_param_vec[2:(1+free_state_size)]
+
+      hmc_step_sizes <- tf$cast(
+        x = tf$reshape(
           hmc_epsilon * (hmc_diag_sd / tf$reduce_sum(hmc_diag_sd)),
-          shape = shape(dim(free_state)[[2]])
-        )
+          shape = shape(free_state_size)
+        ),
+        dtype = tf$float64
       )
-
+      # TF1/2 check
+      # where is "free_state" pulled from, given that it is the
+      # argument to this function, "generate_log_prob_function" ?
       # log probability function
-      tfe$log_prob_fun <- dag$generate_log_prob_function()
 
       # build the kernel
       # nolint start
-      dag$tf_run(
-        sampler_kernel <- tfp$mcmc$HamiltonianMonteCarlo(
-          target_log_prob_fn = log_prob_fun,
-          step_size = hmc_step_sizes,
-          num_leapfrog_steps = hmc_l,
-          seed = rng_seed
-        )
+
+      sampler_kernel <- tfp$mcmc$HamiltonianMonteCarlo(
+        target_log_prob_fn = dag$tf_log_prob_function_adjusted,
+        step_size = hmc_step_sizes,
+        num_leapfrog_steps = hmc_l
+      )
+      return(
+        sampler_kernel
       )
       # nolint end
     },
@@ -853,49 +944,58 @@ rwmh_sampler <- R6Class(
       diag_sd = 1
     ),
     accept_target = 0.44,
-    define_tf_kernel = function() {
+
+    define_tf_kernel = function(sampler_param_vec) {
+
+      # wrap this up into a function to extract these out
+      free_state_size <- length(sampler_param_vec) - 1 # get it from dag object
+      # e.g., length(dag$free_state)
+      rwmh_epsilon <- sampler_param_vec[0]
+      rwmh_diag_sd <- sampler_param_vec[1:(1+free_state_size)]
+
       dag <- self$model$dag
       tfe <- dag$tf_environment
+
       tfe$rwmh_proposal <- switch(self$parameters$proposal,
-        normal = tfp$mcmc$random_walk_normal_fn,
-        uniform = tfp$mcmc$random_walk_uniform_fn
+                                  normal = tfp$mcmc$random_walk_normal_fn,
+                                  uniform = tfp$mcmc$random_walk_uniform_fn
       )
 
-      tfe$log_prob_fun <- dag$generate_log_prob_function()
+      # TF1/2 check
+      # I think a good portion of this code could be abstracted away
+      # Perhaps from `rwmh_epsilon` to `new_state_fn`
+      # could be passed
+      # tfe$log_prob_fun <- dag$generate_log_prob_function()
 
       # tensors for sampler parameters
-      dag$tf_run(
-        rwmh_epsilon <- tf$compat$v1$placeholder(dtype = tf_float())
-      )
+      # rwmh_epsilon <- tf$compat$v1$placeholder(dtype = tf_float())
 
       # need to pass in the value for this placeholder as a matrix (shape(n, 1))
-      dag$tf_run(
-        rwmh_diag_sd <- tf$compat$v1$placeholder(
-          dtype = tf_float(),
-          shape = shape(dim(free_state)[[2]], 1)
-        )
-      )
+      # rwmh_diag_sd <- tf$compat$v1$placeholder(
+      #   dtype = tf_float(),
+      #   # TF1/2 check
+      # again what do we with with `free_state`?
+      #   shape = shape(dim(free_state)[[2]], 1)
+      # )
 
       # but it step_sizes must be a vector (shape(n, )), so reshape it
-      dag$tf_run(
-        rwmh_step_sizes <- tf$reshape(
-          rwmh_epsilon * (rwmh_diag_sd / tf$reduce_sum(rwmh_diag_sd)),
-          shape = shape(dim(free_state)[[2]])
-        )
+      rwmh_step_sizes <- tf$reshape(
+        rwmh_epsilon * (rwmh_diag_sd / tf$reduce_sum(rwmh_diag_sd)),
+        # TF1/2 check
+        # what are we to do about `free_state` here?
+        shape = shape(free_state_size)
       )
 
-      dag$tf_run(
-        new_state_fn <- rwmh_proposal(scale = rwmh_step_sizes)
-      )
+      new_state_fn <- tfe$rwmh_proposal(scale = rwmh_step_sizes)
 
       # build the kernel
       # nolint start
-      dag$tf_run(
-        sampler_kernel <- tfp$mcmc$RandomWalkMetropolis(
-          target_log_prob_fn = log_prob_fun,
-          new_state_fn = new_state_fn,
-          seed = rng_seed
-        )
+      sampler_kernel <- tfp$mcmc$RandomWalkMetropolis(
+        target_log_prob_fn = dag$tf_log_prob_function_adjusted,
+        new_state_fn = new_state_fn
+      )
+      return(
+        sampler_kernel
       )
       # nolint end
     },
@@ -921,38 +1021,26 @@ slice_sampler <- R6Class(
     ),
     tuning_interval = Inf,
     uses_metropolis = FALSE,
-    define_tf_kernel = function() {
+
+    define_tf_kernel = function(sampler_param_vec) {
+      slice_max_doublings <- tensorflow::as_tensor(
+        x = sampler_param_vec[0],
+        dtype = tf$int32
+      )
+
       dag <- self$model$dag
       tfe <- dag$tf_environment
 
-      if (dag$tf_float != "float32") {
-        msg <- cli::format_error(
-          c(
-            "slice sampler can only currently be used for models defined with \\
-            single precision",
-            "set {.code model(..., precision = 'single')} instead"
-          )
-        )
-        stop(
-          msg,
-          call. = FALSE
-        )
-      }
-
-      tfe$log_prob_fun <- dag$generate_log_prob_function()
-      dag$tf_run(
-        slice_max_doublings <- tf$compat$v1$placeholder(dtype = tf$int32)
-      )
-
       # build the kernel
       # nolint start
-      dag$tf_run(
-        sampler_kernel <- tfp$mcmc$SliceSampler(
-          target_log_prob_fn = log_prob_fun,
-          step_size = fl(1),
-          max_doublings = slice_max_doublings,
-          seed = rng_seed
-        )
+      sampler_kernel <- tfp$mcmc$SliceSampler(
+        target_log_prob_fn = dag$tf_log_prob_function_adjusted,
+        step_size = fl(1),
+        max_doublings = slice_max_doublings
+      )
+
+      return(
+        sampler_kernel
       )
       # nolint end
     },
@@ -969,274 +1057,3 @@ slice_sampler <- R6Class(
     }
   )
 )
-
-optimiser <- R6Class(
-  "optimiser",
-  inherit = inference,
-  public = list(
-
-    # optimiser information
-    name = "",
-    method = "method",
-    parameters = list(),
-    other_args = list(),
-    max_iterations = 100L,
-    tolerance = 1e-6,
-    uses_callbacks = TRUE,
-    adjust = TRUE,
-
-    # modified during optimisation
-    it = 0,
-    old_obj = Inf,
-    diff = Inf,
-
-    # set up the model
-    initialize = function(initial_values,
-                          model,
-                          name,
-                          method,
-                          parameters,
-                          other_args,
-                          max_iterations,
-                          tolerance,
-                          adjust) {
-      super$initialize(initial_values,
-        model,
-        parameters = list(),
-        seed = get_seed()
-      )
-
-      self$name <- name
-      self$method <- method
-      self$parameters <- parameters
-      self$other_args <- other_args
-      self$max_iterations <- as.integer(max_iterations)
-      self$tolerance <- tolerance
-      self$adjust <- adjust
-
-      if ("uses_callbacks" %in% names(other_args)) {
-        self$uses_callbacks <- other_args$uses_callbacks
-      }
-
-      self$create_optimiser_objective()
-      self$create_tf_minimiser()
-    },
-    parameter_names = function() {
-      names(self$parameters)
-    },
-    set_dtype = function(parameter_name, dtype) {
-      params <- self$parameters
-      param_names <- self$parameter_names()
-
-      if (parameter_name %in% param_names) {
-        param <- params[[parameter_name]]
-        self$model$dag$on_graph(
-          tf_param <- tf$constant(param, dtype = dtype)
-        )
-        params[[parameter_name]] <- tf_param
-      }
-
-      self$parameters <- params
-    },
-
-    # initialize the variables, then set the ones we care about
-    set_inits = function() {
-      dag <- self$model$dag
-      tfe <- dag$tf_environment
-
-      dag$tf_sess_run(tf$compat$v1$global_variables_initializer())
-
-      shape <- tfe$optimiser_free_state$shape
-      dag$on_graph(
-        tfe$optimiser_init <- tf$constant(self$free_state,
-          shape = shape,
-          dtype = tf_float()
-        )
-      )
-
-      . <- dag$tf_sess_run(optimiser_free_state$assign(optimiser_init))
-    },
-
-    # create a separate free state variable and objective, since optimisers must
-    # use variables
-    create_optimiser_objective = function() {
-      dag <- self$model$dag
-      tfe <- dag$tf_environment
-
-      # define a *variable* free state object
-      if (!live_pointer("optimiser_free_state", envir = tfe)) {
-        dag$define_free_state("variable", name = "optimiser_free_state")
-      }
-
-      # use the log prob function to define objectives from the variable
-      if (!live_pointer("optimiser_objective_adj", envir = tfe)) {
-        log_prob_fun <- dag$generate_log_prob_function(which = "both")
-        dag$on_graph(objectives <- log_prob_fun(tfe$optimiser_free_state))
-
-        assign("optimiser_objective_adj",
-          -objectives$adjusted,
-          envir = tfe
-        )
-
-        assign("optimiser_objective",
-          -objectives$unadjusted,
-          envir = tfe
-        )
-      }
-    },
-    run = function() {
-      self$model$dag$build_feed_dict()
-      self$set_inits()
-      self$run_minimiser()
-      self$fetch_free_state()
-    },
-    fetch_free_state = function() {
-
-      # get the free state as a vector
-      self$free_state <- self$model$dag$tf_sess_run(optimiser_free_state)
-    },
-    return_outputs = function() {
-      dag <- self$model$dag
-
-      # if the optimiser was ignoring the callbacks, we have no idea about the
-      # number of iterations or convergence
-      if (!self$uses_callbacks) {
-        self$it <- NA
-      }
-
-      converged <- self$it < (self$max_iterations - 1)
-
-      par <- dag$trace_values(self$free_state, flatten = FALSE)
-      par <- lapply(par, drop_first_dim)
-      par <- lapply(par, drop_column_dim)
-
-      list(
-        par = par,
-        value = -dag$tf_sess_run(joint_density),
-        iterations = self$it,
-        convergence = ifelse(converged, 0, 1)
-      )
-    }
-  )
-)
-
-tf_optimiser <- R6Class(
-  "tf_optimiser",
-  inherit = optimiser,
-  public = list(
-
-    # some of the optimisers are very fussy about dtypes, so convert them now
-    sanitise_dtypes = function() {
-      self$set_dtype("global_step", tf$int64)
-
-      if (self$name == "proximal_gradient_descent") {
-        lapply(self$parameter_names(), self$set_dtype, tf$float64)
-      }
-
-      if (self$name == "proximal_adagrad") {
-        fussy_params <- c(
-          "learning_rate",
-          "l1_regularization_strength",
-          "l2_regularization_strength"
-        )
-
-        lapply(fussy_params, self$set_dtype, tf$float64)
-      }
-    },
-
-    # create an op to minimise the objective
-    create_tf_minimiser = function() {
-      dag <- self$model$dag
-      tfe <- dag$tf_environment
-
-      self$sanitise_dtypes()
-
-      optimise_fun <- eval(parse(text = self$method))
-      dag$on_graph(tfe$tf_optimiser <- do.call(
-        optimise_fun,
-        self$parameters
-      ))
-
-      if (self$adjust) {
-        dag$tf_run(train <- tf_optimiser$minimize(optimiser_objective_adj))
-      } else {
-        dag$tf_run(train <- tf_optimiser$minimize(optimiser_objective))
-      }
-    },
-
-    # minimise the objective function
-    run_minimiser = function() {
-      self$set_inits()
-
-      while (self$it < self$max_iterations &
-        self$diff > self$tolerance) {
-        self$it <- self$it + 1
-        self$model$dag$tf_sess_run(train)
-        if (self$adjust) {
-          obj <- self$model$dag$tf_sess_run(optimiser_objective_adj)
-        } else {
-          obj <- self$model$dag$tf_sess_run(optimiser_objective)
-        }
-        self$diff <- abs(self$old_obj - obj)
-        self$old_obj <- obj
-      }
-    }
-  )
-)
-
-scipy_optimiser <- R6Class(
-  "scipy_optimiser",
-  inherit = optimiser,
-  public = list(
-    create_tf_minimiser = function() {
-      dag <- self$model$dag
-      tfe <- dag$tf_environment
-
-      opt_fun <- eval(parse(text = "tf$contrib$opt$ScipyOptimizerInterface"))
-
-      if (self$adjust) {
-        loss <- tfe$optimiser_objective_adj
-      } else {
-        loss <- tfe$optimiser_objective
-      }
-
-      args <- list(
-        loss = loss,
-        method = self$method,
-        options = c(self$parameters,
-          maxiter = self$max_iterations
-        ),
-        tol = self$tolerance
-      )
-
-      dag$on_graph(tfe$tf_optimiser <- do.call(opt_fun, args))
-    },
-    obj_progress = function(obj) {
-      self$diff <- abs(self$old_obj - obj)
-      self$old_obj <- obj
-    },
-    it_progress = function(...) {
-      self$it <- self$it + 1
-    },
-    run_minimiser = function() {
-      dag <- self$model$dag
-      tfe <- dag$tf_environment
-      tfe$it_progress <- self$it_progress
-      tfe$obj_progress <- self$obj_progress
-
-      self$set_inits()
-
-      # run the optimiser, suppressing python's yammering
-      quietly(
-        dag$tf_run(
-          tf_optimiser$minimize(sess,
-            feed_dict = feed_dict,
-            step_callback = it_progress,
-            loss_callback = obj_progress,
-            fetches = list(optimiser_objective_adj)
-          )
-        )
-      )
-    }
-  )
-)
