@@ -5,27 +5,30 @@ node <- R6Class(
     unique_name = "",
     parents = list(),
     children = list(),
+    # named greta arrays giving different representations of the greta array
+    # represented by this node that have already been calculated, to be used for
+    # computational speedups or numerical stability. E.g. a logarithm or a
+    # cholesky factor
     representations = list(),
+    anti_representations = list(),
     .value = array(NA),
     dim = NA,
     distribution = NULL,
     initialize = function(dim = NULL, value = NULL) {
-      if (is.null(dim)) {
-        dim <- c(1, 1)
-      }
+      ## browser()
+      dim <- dim %||% c(1,1)
 
       # coerce dim to integer
       dim <- as.integer(dim)
 
       # store array (updates dim)
-      if (is.null(value)) {
-        value <- unknowns(dim = dim)
-      }
+      value <- value %||% unknowns(dim = dim)
 
       self$value(value)
-      self$get_unique_name()
+      self$create_unique_name()
     },
     register = function(dag) {
+      ## TODO add explaining variable
       if (!(self$unique_name %in% names(dag$node_list))) {
         dag$node_list[[self$unique_name]] <- self
       }
@@ -33,6 +36,9 @@ node <- R6Class(
 
     # recursively register self and family
     register_family = function(dag) {
+      ## TF1/2
+      ## Rename with an explaining variable
+      ## TODO add explaining variable
       if (!(self$unique_name %in% names(dag$node_list))) {
 
         # add self to list
@@ -42,7 +48,7 @@ node <- R6Class(
         family <- c(self$list_children(dag), self$list_parents(dag))
 
         # get and assign their names
-        family_names <- vapply(family, member, "unique_name", FUN.VALUE = "")
+        family_names <- extract_unique_names(family)
         names(family) <- family_names
 
         # find the unregistered ones
@@ -69,6 +75,9 @@ node <- R6Class(
       node$remove_child(self)
     },
     list_parents = function(dag) {
+      ## TF1/2
+      ## tf_cholesky
+      ## is there a way here to add some check for cholesky?
       parents <- self$parents
 
       # if this node is being sampled and has a distribution, consider
@@ -78,6 +87,19 @@ node <- R6Class(
         parents <- c(parents, list(self$distribution))
       }
 
+      if (mode == "sampling" & has_representation(self, "cholesky")){
+        # remove cholesky representation node from parents
+        parent_names <- extract_unique_names(parents)
+        antirep_name <- get_node(self$representations$cholesky)$unique_name
+        parent_names_keep <- setdiff(parent_names, antirep_name)
+        parents <- parents[match(parent_names_keep, parent_names)]
+      }
+
+      if (mode == "sampling" & has_anti_representation(self, "chol2symm")){
+        chol2symm_node <- get_node(self$anti_representations$chol2symm)
+        parents <- c(parents, list(chol2symm_node))
+      }
+
       parents
     },
     add_child = function(node) {
@@ -98,11 +120,7 @@ node <- R6Class(
       # that a child node
       mode <- dag$how_to_define(self)
       if (mode == "sampling" & has_distribution(self)) {
-        child_names <- vapply(children,
-          member,
-          "unique_name",
-          FUN.VALUE = character(1)
-        )
+        child_names <- extract_unique_names(children)
         keep <- child_names != self$distribution$unique_name
         children <- children[keep]
       }
@@ -118,11 +136,7 @@ node <- R6Class(
       parents <- self$parents
 
       if (length(parents) > 0) {
-        names <- vapply(parents,
-          member,
-          "unique_name",
-          FUN.VALUE = character(1)
-        )
+        names <- extract_unique_names(parents)
 
         if (recursive) {
           their_parents <- function(x) {
@@ -168,10 +182,11 @@ node <- R6Class(
       tf_name %in% ls(dag$tf_environment)
     },
 
-    # define this and all descendent objects on tensorflow graph in environment
-    # env
+    # define this and all descendent objects on TF graph in environment
     define_tf = function(dag) {
-
+      if (Sys.getenv("GRETA_DEBUG") == "true") {
+        browser()
+      }
       # if defined already, skip
       if (!self$defined(dag)) {
 
@@ -180,16 +195,19 @@ node <- R6Class(
           function(x) x$defined(dag),
           FUN.VALUE = FALSE
         )
-
         if (any(!parents_defined)) {
           parents <- self$list_parents(dag)
           lapply(
             parents[which(!parents_defined)],
-            function(x) x$define_tf(dag)
+            function(x){
+              # browser()
+              x$define_tf(dag)
+            }
           )
         }
 
         # then define self
+          # stop("hi from the future ... parents are of class:", str(parents))
         self$tf(dag)
       }
     },
@@ -217,16 +235,7 @@ node <- R6Class(
     },
     set_distribution = function(distribution) {
 
-      # check it
-      if (!inherits(distribution, "distribution_node")) {
-        msg <- cli::format_error(
-          "invalid distribution"
-        )
-        stop(
-          msg,
-          call. = FALSE
-        )
-      }
+      check_is_distribution_node(distribution)
 
       # add it
       self$distribution <- distribution
@@ -245,7 +254,24 @@ node <- R6Class(
 
       text
     },
-    get_unique_name = function() {
+    cli_description = function() {
+      text <- node_type(self)
+      text <- node_type_colour(text)
+
+      dist_txt <- glue::glue("{self$distribution$distribution_name} distribution")
+      if (has_distribution(self)) {
+        text <- cli::cli_fmt(
+          cli::cli_text(
+            # "{text} following a {.strong {dist_txt}}"
+            "{text} following a {cli::col_yellow({dist_txt})}"
+          )
+        )
+      }
+
+      text
+    },
+
+    create_unique_name = function() {
       self$unique_name <- glue::glue("node_{rhex()}")
     },
     plotting_label = function() {
@@ -265,11 +291,37 @@ node <- R6Class(
       }
 
       label
+    },
+    make_antirepresentations = function(representations){
+      mapply(
+        FUN = self$make_one_anti_representation,
+        representations,
+        names(representations)
+        )
+    },
+    make_one_anti_representation = function(ga, name){
+      node <- get_node(ga)
+      anti_name <- self$find_anti_name(name)
+      node$anti_representations[[anti_name]] <- as.greta_array(self)
+      node
+    },
+    find_anti_name = function(name){
+      switch(name,
+             cholesky = "chol2symm",
+             chol2symm = "chol",
+             exp = "log",
+             log = "exp",
+             probit = "iprobit",
+             iprobit = "probit",
+             logit = "ilogit",
+             ilogit = "logit"
+      )
     }
   )
 )
 
-# generic to grab dimensions
+#' @title generic to grab dimensions of nodes
+#' @param x greta node class
 #' @export
 dim.node <- function(x) {
   x$dim
@@ -277,19 +329,16 @@ dim.node <- function(x) {
 
 # coerce an object to a node
 to_node <- function(x) {
-  if (!inherits(x, "node")) {
-    if (inherits(x, "greta_array")) {
+  # TODO: clean up this logic
+  if (!is.node(x)) {
+    if (is.greta_array(x)) {
       x <- get_node(x)
     } else if (is.numeric(x)) {
       x <- data_node$new(x)
     } else {
-      msg <- cli::format_error(
+      cli::cli_abort(
         "cannot coerce object to a node"
       )
-      stop(
-        msg,
-        call. = FALSE
-      )
     }
   }
   x
