@@ -45,16 +45,8 @@
 #' }
 variable <- function(lower = -Inf, upper = Inf, dim = NULL) {
   check_tf_version("error")
-
-  if (inherits(lower, "greta_array") | inherits(upper, "greta_array")) {
-    msg <- cli::format_error(
-      "lower and upper must be fixed, they cannot be another greta array"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_param_greta_array(lower)
+  check_param_greta_array(upper)
 
   node <- variable_node$new(lower, upper, dim)
   as.greta_array(node)
@@ -80,39 +72,17 @@ cholesky_variable <- function(dim, correlation = FALSE) {
   n_dim <- length(dim)
   if (n_dim == 1) {
     dim <- c(dim, dim)
-  } else if (n_dim == 2) {
-    if (dim[1] != dim[2]) {
-      msg <- cli::format_error(
-        c(
-          "cholesky variables must be square",
-          "However its dimension is: {.val {paste(dim, collapse = 'x')}}"
-        )
-      )
-      stop(
-        msg,
-        call. = FALSE
-      )
-    }
-  } else {
-    msg <- cli::format_error(
-      c(
-        "{.arg dim} can either be a scalar or a vector of length 2",
-        "However {.arg dim} has length {.val {length(dim)}}, and contains: \\
-        {.val {paste(dim, collapse = ', ')}}"
-      )
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
   }
 
+  check_square(dim = dim)
+  check_dim_length(dim)
+
   k <- dim[1]
 
   # dimension of the free state version
   free_dim <- ifelse(correlation,
-    k * (k - 1) / 2,
-    k + k * (k - 1) / 2
+                     k * (k - 1) / 2,
+                     k + k * (k - 1) / 2
   )
 
   # create variable node
@@ -124,8 +94,8 @@ cholesky_variable <- function(dim, correlation = FALSE) {
 
   # set the constraint, to enable transformation
   node$constraint <- ifelse(correlation,
-    "correlation_matrix",
-    "covariance_matrix"
+                            "correlation_matrix",
+                            "covariance_matrix"
   )
 
   # set the printed value to be nicer
@@ -154,21 +124,10 @@ simplex_variable <- function(dim) {
   }
 
   dim <- check_dims(target_dim = dim)
-
-  # dimension of the free state version
   n_dim <- length(dim)
   last_dim <- dim[n_dim]
-  if (!last_dim > 1) {
-    msg <- cli::format_error(
-      "the final dimension of a simplex variable must have more than one \\
-      element",
-      "The final dimension has: {.val {length(last_dim)} elements}"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  # dimension of the free state version
+  check_final_dim(dim, thing = "simplex variable")
 
   raw_dim <- dim
   raw_dim[n_dim] <- last_dim - 1
@@ -206,19 +165,7 @@ ordered_variable <- function(dim) {
 
   dim <- check_dims(target_dim = dim)
 
-  # dimension of the free state version
-  n_dim <- length(dim)
-  if (!dim[n_dim] > 1) {
-    msg <- cli::format_error(
-      "the final dimension of an ordered variable must have more than \\
-      one element",
-      "the final dimension has: {.val {length(last_dim)} elements}"
-    )
-    stop(
-      msg,
-      call. = FALSE
-    )
-  }
+  check_final_dim(dim, thing = "ordered variable")
 
   # create variable node
   node <- vble(truncation = c(-Inf, Inf), dim = dim)
