@@ -20,12 +20,13 @@ grab <- function(x, dag = NULL) {
   if (inherits(x, "greta_array")) {
     node <- get_node(x)
     dag <- dag_class$new(list(x))
-    dag$define_tf()
   }
 
-  dag$set_tf_data_list("batch_size", 1L)
-  dag$build_feed_dict()
-  out <- dag$tf_sess_run(dag$tf_name(node), as_text = TRUE)
+  dag$tf_environment$batch_size <- 1L
+  node$define_tf(dag)
+  x_name <- dag$tf_name(node)
+  out <- dag$tf_environment[[x_name]]
+  out <- as.array(out)
   drop_first_dim(out)
 }
 
@@ -55,9 +56,15 @@ get_density <- function(distrib, data) {
   as.vector(grab(tensor, dag))
 }
 
-compare_distribution <- function(greta_fun, r_fun, parameters, x,
-                                 dim = NULL, multivariate = FALSE,
-                                 tolerance = 1e-4) {
+compare_distribution <- function(
+  greta_fun,
+  r_fun,
+  parameters,
+  x,
+  dim = NULL,
+  multivariate = FALSE,
+  tolerance = 1e-4
+) {
   # calculate the absolute difference in the log density of some data between
   # greta and a r benchmark.
   # 'greta_fun' is the greta distribution constructor function (e.g. normal())
@@ -68,8 +75,11 @@ compare_distribution <- function(greta_fun, r_fun, parameters, x,
 
   # define greta distribution, with fixed values
   greta_log_density <- greta_density(
-    greta_fun, parameters, x,
-    dim, multivariate
+    greta_fun,
+    parameters,
+    x,
+    dim,
+    multivariate
   )
   # get R version
   r_log_density <- log(do.call(r_fun, c(list(x), parameters)))
@@ -80,11 +90,15 @@ compare_distribution <- function(greta_fun, r_fun, parameters, x,
 
 # evaluate the log density of x, given 'parameters' and a distribution
 # constructor function 'fun'
-greta_density <- function(fun, parameters, x,
-                          dim = NULL, multivariate = FALSE) {
-  if (is.null(dim)) {
-    dim <- NROW(x)
-  }
+greta_density <- function(
+  fun,
+  parameters,
+  x,
+  dim = NULL,
+  multivariate = FALSE
+) {
+
+  dim <- dim %||% NROW(x)
 
   # add the output dimension to the arguments list
   dim_list <- list(dim = dim)
@@ -114,18 +128,13 @@ greta_density <- function(fun, parameters, x,
 
   # create dag
   dag <- dag_class$new(list(x_))
-  dag$define_tf()
-  dag$set_tf_data_list("batch_size", 1L)
-  dag$build_feed_dict()
+
+  dag$tf_environment$batch_size <- 1L
+  distrib_node$define_tf(dag)
 
   # get the log density as a vector
-  dag$on_graph(
-    result <- dag$evaluate_density(distrib_node, get_node(x_))
-  )
-  assign("test_density", result, dag$tf_environment)
-
-  density <- dag$tf_sess_run(test_density)
-  as.vector(density)
+  result <- dag$evaluate_density(distrib_node, get_node(x_))
+  as.vector(result)
 }
 
 # execute a call via greta, swapping the objects named in 'swap' to greta
@@ -139,7 +148,8 @@ with_greta <- function(call, swap = c("x"), swap_scope = 1) {
     paste(swap_entries, collapse = ", "),
     ")"
   )
-  swap_list <- eval(parse(text = swap_text),
+  swap_list <- eval(
+    parse(text = swap_text),
     envir = parent.frame(n = swap_scope)
   )
 
@@ -165,7 +175,8 @@ check_expr <- function(expr, swap = c("x"), tolerance = 1e-4) {
   call <- substitute(expr)
 
   r_out <- eval(expr)
-  greta_out <- with_greta(call,
+  greta_out <- with_greta(
+    call,
     swap = swap,
     swap_scope = 2
   )
@@ -175,9 +186,11 @@ check_expr <- function(expr, swap = c("x"), tolerance = 1e-4) {
 
 # generate a random string to describing a binary operation on two variables, do
 # an op selected from 'ops' to an arg from 'args' and to init
-add_op_string <- function(init = "a",
-                          args = c("a", "b"),
-                          ops = c("+", "-", "*", "/")) {
+add_op_string <- function(
+  init = "a",
+  args = c("a", "b"),
+  ops = c("+", "-", "*", "/")
+) {
   op <- sample(ops, 1)
   arg <- sample(args, 1)
   sprintf("(%s %s %s)", arg, op, init)
@@ -198,9 +211,13 @@ gen_opfun <- function(n, ops) {
 
 # sample n values from a distribution by HMC, check they all have the correct
 # support greta array is defined as a stochastic in the call
-sample_distribution <- function(greta_array, n = 10,
-                                lower = -Inf, upper = Inf,
-                                warmup = 1) {
+sample_distribution <- function(
+  greta_array,
+  n = 10,
+  lower = -Inf,
+  upper = Inf,
+  warmup = 1
+) {
   m <- model(greta_array, precision = "double")
   draws <- mcmc(m, n_samples = n, warmup = warmup, verbose = FALSE)
   samples <- as.matrix(draws)
@@ -217,11 +234,13 @@ sample_distribution <- function(greta_array, n = 10,
   expect_true(all(above_lower & below_upper))
 }
 
-compare_truncated_distribution <- function(greta_fun,
-                                           which,
-                                           parameters,
-                                           truncation,
-                                           tolerance = 1e-4) {
+compare_truncated_distribution <- function(
+  greta_fun,
+  which,
+  parameters,
+  truncation,
+  tolerance = 1e-4
+) {
   # calculate the absolute difference in the log density of some data between
   # greta and a r benchmark, for an implied truncated distribution 'greta_array'
   # is a greta array created from a distribution and a constrained variable
@@ -316,8 +335,11 @@ get_output <- function(expr) {
 
 # mock up mcmc progress bar output for neurotic testing
 mock_mcmc <- function(n_samples = 1010) {
-  pb <- create_progress_bar("sampling", c(0, n_samples),
-    pb_update = 10, width = 50
+  pb <- create_progress_bar(
+    "sampling",
+    c(0, n_samples),
+    pb_update = 10,
+    width = 50
   )
   iterate_progress_bar(pb, n_samples, rejects = 10, chains = 1)
 }
@@ -362,6 +384,41 @@ rlkjcorr <- function(n, eta = 1, dimension = 2) {
   r
 }
 
+# normalising component of lkj (depends only on eta and dimension)
+# NOTE
+# we may need to find a better way to do the normalising of this/find a better
+# reference equation. E.g., looking in
+# potentially our implementation here is incorrect. This is not impacting
+# IID sampling or MCMC, but is currently causing some tests to fail where we
+# are ensuring our densities are accurate (in test_distributions.R)
+lkj_log_normalising <- function(eta, n) {
+  log_pi <- log(pi)
+  ans <- 0
+  for (k in 1:(n - 1)) {
+    ans <- ans + log_pi * (k / 2)
+    ans <- ans + lgamma(eta + (n - 1 - k) / 2)
+    ans <- ans - lgamma(eta + (n - 1) / 2)
+  }
+  ans
+}
+
+dlkj_correlation_unnormalised <- function(x, eta, log = FALSE, dimension = NULL) {
+  res <- (eta - 1) * log(det(x))
+  if (!log) {
+    res <- exp(res)
+  }
+  res
+}
+
+# lkj density
+dlkj_correlation <- function(x, eta, log = FALSE, dimension = NULL) {
+  res <- (eta - 1) * log(det(x)) - lkj_log_normalising(eta, ncol(x))
+  if (!log) {
+    res <- exp(res)
+  }
+  res
+}
+
 # helper RNG functions
 rmvnorm <- function(n, mean, Sigma) { # nolint
   mvtnorm::rmvnorm(n = n, mean = mean, sigma = Sigma)
@@ -528,11 +585,13 @@ get_upper_tri <- function(x, diag) {
 
 # compare iid samples from a greta distribution (using calculate) against a
 # comparison R RNG function
-compare_iid_samples <- function(greta_fun,
-                                r_fun,
-                                parameters,
-                                nsim = 200,
-                                p_value_threshold = 0.001) {
+compare_iid_samples <- function(
+  greta_fun,
+  r_fun,
+  parameters,
+  nsim = 200,
+  p_value_threshold = 0.001
+) {
   greta_array <- do.call(greta_fun, parameters)
 
   # get information about distribution
@@ -546,7 +605,6 @@ compare_iid_samples <- function(greta_fun,
 
   # reshape to matrix or vector
   if (multivariate) {
-
     # if it's a symmetric matrix, take only a triangle and flatten it
     if (name %in% c("wishart", "lkj_correlation")) {
       include_diag <- name == "wishart"
@@ -589,11 +647,16 @@ skip_if_not_release <- function() {
 # the two IID random number generators for the data generating function
 # ('p_theta' = generator for the prior, 'p_x_bar_theta' = generator for the
 # likelihood), 'niter' the number of MCMC samples to compare
-check_geweke <- function(sampler, model, data,
-                         p_theta, p_x_bar_theta,
-                         niter = 2000, warmup = 1000,
-                         title = "Geweke test") {
-
+check_geweke <- function(
+  sampler,
+  model,
+  data,
+  p_theta,
+  p_x_bar_theta,
+  niter = 2000,
+  warmup = 1000,
+  thin = 1
+) {
   # sample independently
   target_theta <- p_theta(niter)
 
@@ -608,32 +671,51 @@ check_geweke <- function(sampler, model, data,
     warmup = warmup
   )
 
+  geweke_checks <- list(
+    target_theta = do_thinning(target_theta, thin),
+    greta_theta = do_thinning(greta_theta, thin)
+  )
+
+  geweke_checks
+
+}
+
+geweke_qq <- function(geweke_checks, title){
   # visualise correspondence
   quants <- (1:99) / 100
-  q1 <- stats::quantile(target_theta, quants)
-  q2 <- stats::quantile(greta_theta, quants)
+  q1 <- stats::quantile(geweke_checks$target_theta, quants)
+  q2 <- stats::quantile(geweke_checks$greta_theta, quants)
   plot(q2, q1, main = title)
   graphics::abline(0, 1)
 
+}
+
+geweke_ks <- function(geweke_checks){
   # do a formal hypothesis test
-  suppressWarnings(stat <- stats::ks.test(target_theta, greta_theta))
-  testthat::expect_gte(stat$p.value, 0.005)
+  suppressWarnings(stat <- stats::ks.test(geweke_checks$target_theta,
+                                          geweke_checks$greta_theta))
+  stat
 }
 
 # sample from a prior on theta the long way round, fro use in a Geweke test:
 # gibbs sampling the posterior p(theta | x) and the data generating function p(x
 # | theta). Only retain the samples of theta from the joint distribution,
-p_theta_greta <- function(niter, model, data,
-                          p_theta, p_x_bar_theta,
-                          sampler = hmc(),
-                          warmup = 1000) {
-
+p_theta_greta <- function(
+  niter,
+  model,
+  data,
+  p_theta,
+  p_x_bar_theta,
+  sampler = hmc(),
+  warmup = 1000
+) {
   # set up and initialize trace
   theta <- rep(NA, niter)
   theta[1] <- p_theta(1)
 
   # set up and tune sampler
-  draws <- mcmc(model,
+  draws <- mcmc(
+    model,
     warmup = warmup,
     n_samples = 1,
     chains = 1,
@@ -641,36 +723,52 @@ p_theta_greta <- function(niter, model, data,
     verbose = FALSE
   )
 
+  # set up a progress bar and do a first increment
+  cli::cli_progress_bar("Geweke test iterations", total = niter)
+  cli::cli_progress_update()
+
   # now loop through, sampling and updating x and returning theta
   for (i in 2:niter) {
 
+    # update the progress bar
+    cli::cli_progress_update()
+
     # sample x given theta
     x <- p_x_bar_theta(theta[i - 1])
 
-    # put x in the data list
+    # replace x in the node
     dag <- model$dag
-    target_name <- dag$tf_name(get_node(data))
-    x_array <- array(x, dim = c(1, dim(data)))
-    dag$tf_environment$data_list[[target_name]] <- x_array
+    x_node <- get_node(data)
+    x_node$value(as.matrix(x))
 
-    # put theta in the free state
+    # rewrite the log prob tf function, and the tf function for the posterior
+    # samples, now using this value of x (slow, but necessary in eager mode)
+    dag$tf_log_prob_function <- NULL
+    dag$define_tf_log_prob_function()
     sampler <- attr(draws, "model_info")$samplers[[1]]
-    sampler$free_state <- as.matrix(theta[i - 1])
+    sampler$define_tf_evaluate_sample_batch()
 
-    draws <- extra_samples(draws,
+    # take anoteher sample
+    draws <- extra_samples(
+      draws,
       n_samples = 1,
       verbose = FALSE
     )
 
+    # trace the sample
     theta[i] <- tail(as.numeric(draws[[1]]), 1)
+
   }
 
+  # kill the progress_bar
+  cli::cli_progress_done()
+
   theta
 }
 
 # test mcmc for models with analytic posteriors
 
-not_finished <- function(draws, target_samples = 5000) {
+need_more_samples <- function(draws, target_samples = 5000) {
   neff <- coda::effectiveSize(draws)
   rhats <- coda::gelman.diag(
     x = draws,
@@ -689,34 +787,39 @@ new_samples <- function(draws, target_samples = 5000) {
   1.2 * (target_samples - neff) / efficiency
 }
 
-not_timed_out <- function(start_time, time_limit = 300) {
+still_have_time <- function(start_time, time_limit = 300) {
   elapsed <- Sys.time() - start_time
   elapsed < time_limit
 }
 
-get_enough_draws <- function(model,
-                             sampler = sampler,
-                             n_effective = 5000,
-                             time_limit = 300,
-                             verbose = TRUE,
-                             one_by_one = FALSE) {
+get_enough_draws <- function(
+  model,
+  sampler = sampler,
+  n_effective = 5000,
+  time_limit = 300,
+  verbose = TRUE,
+  one_by_one = FALSE
+) {
   start_time <- Sys.time()
-  draws <- mcmc(model,
+  draws <- mcmc(
+    model,
     sampler = sampler,
     verbose = verbose,
     one_by_one = one_by_one
   )
 
-  while (not_finished(draws, n_effective) &
-    not_timed_out(start_time, time_limit)) {
+  while (need_more_samples(draws, n_effective) &&
+         still_have_time(start_time, time_limit)) {
     n_samples <- new_samples(draws, n_effective)
-    draws <- extra_samples(draws, n_samples,
+    draws <- extra_samples(
+      draws,
+      n_samples,
       verbose = verbose,
       one_by_one = one_by_one
     )
   }
 
-  if (not_finished(draws, n_effective)) {
+  if (need_more_samples(draws, n_effective)) {
     stop("could not draws enough effective samples within the time limit")
   }
 
@@ -761,14 +864,14 @@ scaled_error <- function(draws, expectation) {
 # given a sampler (e.g. hmc()) and minimum number of effective samples, ensure
 # that the sampler can draw correct samples from a bivariate normal distribution
 check_mvn_samples <- function(sampler, n_effective = 3000) {
-
   # get multivariate normal samples
   mu <- as_data(t(rnorm(2, 0, 5)))
   sigma <- stats::rWishart(1, 3, diag(2))[, , 1]
   x <- multivariate_normal(mu, sigma)
   m <- model(x, precision = "single")
 
-  draws <- get_enough_draws(m,
+  draws <- get_enough_draws(
+    m,
     sampler = sampler,
     n_effective = n_effective,
     verbose = FALSE
@@ -796,42 +899,168 @@ check_mvn_samples <- function(sampler, n_effective = 3000) {
   # away from truth. There's a 1/100 chance of any one of these scaled errors
   # being greater than qnorm(0.99) if the sampler is correct
   errors <- scaled_error(stat_draws, stat_truth)
-  expect_lte(max(errors), stats::qnorm(0.99))
+  errors
 }
 
+do_thinning <- function(x, thinning = 1) {
+  idx <- seq(1, length(x), by = thinning)
+  x[idx]
+}
+
+
+get_distribution_name <- function(x){
+  x_node <- get_node(x)
+  if (inherits(x_node, "operation_node")){
+    dist_name <- x_node$parents[[1]]$distribution$distribution_name
+  } else {
+    dist_name <- get_node(x)$distribution$distribution_name
+  }
+  dist_name
+}
+
+
 # sample values of greta array 'x' (which must follow a distribution), and
 # compare the samples with iid samples returned by iid_function (which takes the
 # number of arguments as its sole argument), producing a labelled qqplot, and
 # running a KS test for differences between the two samples
-check_samples <- function(x,
-                          iid_function,
-                          sampler = hmc(),
-                          n_effective = 3000,
-                          title = NULL,
-                          one_by_one = FALSE) {
+check_samples <- function(
+  x,
+  iid_function,
+  sampler = hmc(),
+  n_effective = 3000,
+  title = NULL,
+  one_by_one = FALSE,
+  time_limit = 300
+) {
   m <- model(x, precision = "single")
-  draws <- get_enough_draws(m,
+  draws <- get_enough_draws(
+    model = m,
     sampler = sampler,
     n_effective = n_effective,
-    verbose = FALSE,
-    one_by_one = one_by_one
+    verbose = TRUE,
+    one_by_one = one_by_one,
+    time_limit = time_limit
   )
 
   neff <- coda::effectiveSize(draws)
   iid_samples <- iid_function(neff)
   mcmc_samples <- as.matrix(draws)
 
-  # plot
-  if (is.null(title)) {
-    distrib <- get_node(x)$distribution$distribution_name
-    sampler_name <- class(sampler)[1]
-    title <- paste(distrib, "with", sampler_name)
-  }
+  thin_amount <- find_thinning(draws)
 
-  stats::qqplot(mcmc_samples, iid_samples, main = title)
-  graphics::abline(0, 1)
+  mcmc_samples <- do_thinning(mcmc_samples, thin_amount)
+  iid_samples <- do_thinning(iid_samples, thin_amount)
 
+  list(
+    mcmc_samples = mcmc_samples,
+    iid_samples = iid_samples,
+    distrib = get_distribution_name(x),
+    sampler_name = class(sampler)[1]
+  )
+}
+
+qqplot_checked_samples <- function(checked_samples, title){
+
+  distrib <- checked_samples$distrib
+  sampler_name <- checked_samples$sampler_name
+  title <- paste(distrib, "with", sampler_name)
+
+  mcmc_samples <- checked_samples$mcmc_samples
+  iid_samples <- checked_samples$iid_samples
+
+  stats::qqplot(
+    x = mcmc_samples,
+    y = iid_samples,
+    main = title
+    )
+
+  graphics::abline(0, 1)
+}
+
+## helpers for running Kolmogorov-Smirnov test for MCMC samples vs IID samples
+ks_test_mcmc_vs_iid <- function(checked_samples){
   # do a formal hypothesis test
-  suppressWarnings(stat <- ks.test(mcmc_samples, iid_samples))
-  testthat::expect_gte(stat$p.value, 0.01)
+  suppressWarnings(stat <- ks.test(checked_samples$mcmc_samples,
+                                   checked_samples$iid_samples))
+  stat
+}
+
+## helpers for looping through optimisers
+run_opt <- function(
+  m,
+  optmr,
+  max_iterations = 200
+) {
+  opt(
+    m,
+    optimiser = optmr(),
+    max_iterations = max_iterations
+  )
+}
+
+possibly_run_opt <- purrr::possibly(.f = run_opt, otherwise = "error")
+
+opt_df_run <- function(optimisers, m, x) {
+  opt_df <- tibble::enframe(
+    x = optimisers,
+    name = "opt",
+    value = "opt_fn"
+  ) %>%
+    dplyr::mutate(
+      result = lapply(opt_fn, possibly_run_opt, m = m),
+      x_val = list(x)
+    )
+
+  opt_df
+}
+
+tidy_optimisers <- function(opt_df, tolerance = 1e-2) {
+  opt_df %>%
+    dplyr::select(-opt_fn) %>%
+    tidyr::unnest_wider(col = c(result)) %>%
+    dplyr::mutate(
+      par = unname(purrr::flatten(par)),
+      par_x_diff = purrr::map2(
+        .x = par,
+        .y = x_val,
+        .f = function(.x, .y){
+          abs(.y - .x)
+      }),
+      close_to_truth = purrr::map_lgl(
+        par_x_diff,
+        function(x) all(x < tolerance)
+      )
+    ) %>%
+    dplyr::relocate(
+      close_to_truth,
+      par_x_diff,
+      iterations,
+      convergence,
+      .after = opt
+    )
+}
+
+
+# find a thinning rate that sufficiently reduces autocorrelation in the samples
+# example
+# x <- normal(0, 1)
+# m <- model(x)
+# draws <- mcmc(m)
+# find_thinning(draws)
+find_thinning <- function(draws, max_thin = 100, autocorr_threshold = 0.01) {
+  autocorr_list <- coda::autocorr(draws, lags = seq_len(max_thin))
+  autocorrs <- do.call(cbind, autocorr_list)
+  mean_autocorr <- rowMeans(autocorrs)
+  smallest_thin <- which(mean_autocorr < autocorr_threshold)[1]
+  if (is.na(smallest_thin)) {
+    smallest_thin <- max_thin
+    cli::cli_warn(
+      c(
+        "Could not find a thinning value that reduces mean autocorrelation \\
+        below the threshold, {.val {autocorr_threshold}}.",
+        "Using the maximum thinning amount: {.val {max_thin}}"
+      )
+    )
+  }
+  smallest_thin
 }
