@@ -1,7 +1,6 @@
 test_that("as_data coerces correctly", {
   skip_if_not(check_tf_version())
 
-
   # logical, integer and numeric
   # vector, matrix, array, dataframe
 
@@ -108,15 +107,18 @@ test_that("as_data errors informatively", {
 
 
   # wrong class of object
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(NULL)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(list())
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(environment())
   )
 
@@ -127,23 +129,28 @@ test_that("as_data errors informatively", {
   cha_df <- as.data.frame(cha_mat, stringsAsFactors = FALSE)
   cha_df2 <- as.data.frame(cha_mat, stringsAsFactors = TRUE)
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(cha_vec)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(cha_mat)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(cha_arr)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(cha_df)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(cha_df2)
   )
 
@@ -155,15 +162,18 @@ test_that("as_data errors informatively", {
   arr_na <- randn(3, 3)
   arr_na[1, 3] <- NA
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(arr_inf)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(arr_minf)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(arr_na)
   )
 
@@ -171,10 +181,12 @@ test_that("as_data errors informatively", {
   stoch <- normal(0, 1, dim = c(2, 3))
   op <- stoch^2
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(stoch)
   )
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     as_data(op)
   )
 })
