@@ -1,6 +1,5 @@
 test_that("deterministic calculate works with correct lists", {
   skip_if_not(check_tf_version())
-  
 
   # unknown variable
   x <- as_data(c(1, 2))
@@ -28,7 +27,6 @@ test_that("deterministic calculate works with correct lists", {
 
 test_that("stochastic calculate works with correct lists", {
   skip_if_not(check_tf_version())
-  
 
   # nolint start
   # with y ~ N(100, 1 ^ 2), it should be very unlikely that y <= 90
@@ -96,13 +94,13 @@ test_that("stochastic calculate works with correct lists", {
       x = rep(2, k)
     )
   )
+
   expect_true(all(apply(sims$y, 1:2, sum) == 1))
   expect_equal(dim(sims$y), c(nsim, dim(y)))
 })
 
 test_that("deterministic calculate works with greta_mcmc_list objects", {
   skip_if_not(check_tf_version())
-  
 
   samples <- 10
   x <- as_data(c(1, 2))
@@ -132,7 +130,6 @@ test_that("deterministic calculate works with greta_mcmc_list objects", {
 
 test_that("calculate with greta_mcmc_list doesn't mix up variables", {
   skip_if_not(check_tf_version())
-  
 
   a <- normal(-100, 0.001)
   b <- normal(100, 0.001)
@@ -153,7 +150,6 @@ test_that("calculate with greta_mcmc_list doesn't mix up variables", {
 
 test_that("calculate with greta_mcmc_list doesn't lose track of new nodes", {
   skip_if_not(check_tf_version())
-  
 
   z <- normal(0, 1)
   m <- model(z)
@@ -170,7 +166,6 @@ test_that("calculate with greta_mcmc_list doesn't lose track of new nodes", {
 
 test_that("stochastic calculate works with greta_mcmc_list objects", {
   skip_if_not(check_tf_version())
-  
 
   samples <- 10
   chains <- 2
@@ -190,7 +185,7 @@ test_that("stochastic calculate works with greta_mcmc_list objects", {
   )
 
   # this should error without nsim being specified (y is stochastic)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_a <- calculate(a, y, values = draws)
   )
 
@@ -225,7 +220,6 @@ test_that("stochastic calculate works with greta_mcmc_list objects", {
 
 test_that("calculate errors if the mcmc samples unrelated to target", {
   skip_if_not(check_tf_version())
-  
 
   samples <- 10
   chains <- 2
@@ -246,14 +240,13 @@ test_that("calculate errors if the mcmc samples unrelated to target", {
 
   c <- normal(0, 1)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_c <- calculate(c, values = draws)
   )
 })
 
 test_that("stochastic calculate works with mcmc samples & new stochastics", {
   skip_if_not(check_tf_version())
-  
 
   samples <- 10
   chains <- 2
@@ -277,7 +270,7 @@ test_that("stochastic calculate works with mcmc samples & new stochastics", {
 
   # this should error without nsim being specified (b is stochastic and not
   # given by draws)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_b <- calculate(b, values = draws)
   )
 
@@ -288,19 +281,18 @@ test_that("stochastic calculate works with mcmc samples & new stochastics", {
 
 test_that("calculate errors nicely if non-greta arrays are passed", {
   skip_if_not(check_tf_version())
-  
 
   x <- c(1, 2)
   a <- normal(0, 1)
   y <- a * x
 
   # it should error nicely
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, x, values = list(x = c(2, 1)))
   )
 
   # and a hint for this common error
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, list(x = c(2, 1)))
   )
 
@@ -308,14 +300,13 @@ test_that("calculate errors nicely if non-greta arrays are passed", {
 
 test_that("calculate errors nicely if values for stochastics not passed", {
   skip_if_not(check_tf_version())
-  
 
   x <- as_data(c(1, 2))
   a <- normal(0, 1)
   y <- a * x
 
   # it should error nicely
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, values = list(x = c(2, 1)))
   )
 
@@ -325,21 +316,19 @@ test_that("calculate errors nicely if values for stochastics not passed", {
 
 test_that("calculate errors nicely if values have incorrect dimensions", {
   skip_if_not(check_tf_version())
-  
 
   x <- as_data(c(1, 2))
   a <- normal(0, 1)
   y <- a * x
 
   # it should error nicely
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, values = list(a = c(1, 1)))
   )
 })
 
 test_that("calculate works with variable batch sizes", {
   skip_if_not(check_tf_version())
-  
 
   samples <- 100
   x <- as_data(c(1, 2))
@@ -367,7 +356,6 @@ test_that("calculate works with variable batch sizes", {
 
 test_that("calculate errors nicely with invalid batch sizes", {
   skip_if_not(check_tf_version())
-  
 
   samples <- 100
   x <- as_data(c(1, 2))
@@ -377,20 +365,19 @@ test_that("calculate errors nicely with invalid batch sizes", {
   draws <- mcmc(m, warmup = 0, n_samples = samples, verbose = FALSE)
 
   # variable valid batch sizes
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, values = draws, trace_batch_size = 0)
   )
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, values = draws, trace_batch_size = NULL)
   )
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_y <- calculate(y, values = draws, trace_batch_size = NA)
   )
 })
 
 test_that("calculate returns a named list", {
   skip_if_not(check_tf_version())
-  
 
   a <- as_data(randn(3))
   b <- a^2
@@ -415,7 +402,6 @@ test_that("calculate returns a named list", {
 
 test_that("calculate produces the right number of samples", {
   skip_if_not(check_tf_version())
-  
 
   # fix variable
   a <- normal(0, 1)
@@ -434,67 +420,11 @@ test_that("calculate produces the right number of samples", {
   sims <- calculate(y, nsim = 19)
   expect_equal(dim(sims$y), c(19, dim(y)))
 
-  # the global RNG seed should not change if the seed *is* specified
-  before <- rng_seed()
-  sims <- calculate(y, nsim = 1, seed = 12345)
-  after <- rng_seed()
-  expect_identical(before, after)
-
-  # the samples should differ if the seed is *not* specified
-  one <- calculate(y, nsim = 1)
-  two <- calculate(y, nsim = 1)
-  expect_false(identical(one, two))
-
-  # the samples should differ if the seeds are specified differently
-  one <- calculate(y, nsim = 1, seed = 12345)
-  two <- calculate(y, nsim = 1, seed = 54321)
-  expect_false(identical(one, two))
-
-  # the samples should be the same if the seed is the same
-  one <- calculate(y, nsim = 1, seed = 12345)
-  two <- calculate(y, nsim = 1, seed = 12345)
-  expect_identical(one, two)
-})
-
-test_that("calculate uses the local RNG seed", {
-  skip_if_not(check_tf_version())
-  
-
-  # fix variable
-  a <- normal(0, 1)
-  y <- normal(a, 1)
-
-  # the global RNG seed should change if the seed is *not* specified
-  before <- rng_seed()
-  sims <- calculate(y, nsim = 1)
-  after <- rng_seed()
-  expect_false(identical(before, after))
-
-  # the global RNG seed should not change if the seed *is* specified
-  before <- rng_seed()
-  sims <- calculate(y, nsim = 1, seed = 12345)
-  after <- rng_seed()
-  expect_identical(before, after)
-
-  # the samples should differ if the seed is *not* specified
-  one <- calculate(y, nsim = 1)
-  two <- calculate(y, nsim = 1)
-  expect_false(identical(one, two))
-
-  # the samples should differ if the seeds are specified differently
-  one <- calculate(y, nsim = 1, seed = 12345)
-  two <- calculate(y, nsim = 1, seed = 54321)
-  expect_false(identical(one, two))
-
-  # the samples should be the same if the seed is the same
-  one <- calculate(y, nsim = 1, seed = 12345)
-  two <- calculate(y, nsim = 1, seed = 12345)
-  expect_identical(one, two)
 })
 
 test_that("calculate works if distribution-free variables are fixed", {
   skip_if_not(check_tf_version())
-  
+
 
   # fix variable
   a <- variable()
@@ -506,42 +436,39 @@ test_that("calculate works if distribution-free variables are fixed", {
 
 test_that("calculate errors if distribution-free variables are not fixed", {
   skip_if_not(check_tf_version())
-  
 
   # fix variable
   a <- variable()
   y <- normal(a, 1)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_a <- calculate(a, y, nsim = 1)
   )
 })
 
 test_that("calculate errors if a distribution cannot be sampled from", {
   skip_if_not(check_tf_version())
-  
 
   # fix variable
   y <- hypergeometric(5, 3, 2)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     sims <- calculate(y, nsim = 1)
   )
 })
 
 test_that("calculate errors nicely if nsim is invalid", {
   skip_if_not(check_tf_version())
-  
 
   x <- normal(0, 1)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_x <- calculate(x, nsim = 0)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_x <- calculate(x, nsim = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     calc_x <- calculate(x, nsim = "five")
   )
 })
