@@ -1,8 +1,8 @@
 test_that("normal distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::normal,
+  compare_distribution(
+    greta::normal,
     stats::dnorm,
     parameters = list(mean = -2, sd = 3),
     x = rnorm(100, -2, 3)
@@ -12,12 +12,13 @@ test_that("normal distribution has correct density", {
 
 test_that("multidimensional normal distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::normal,
+  compare_distribution(
+    greta::normal,
     stats::dnorm,
     parameters = list(mean = -2, sd = 3),
-    x = array(rnorm(100, -2, 3),
+    x = array(
+      rnorm(100, -2, 3),
       dim = c(10, 2, 5)
     ),
     dim = c(10, 2, 5)
@@ -26,9 +27,9 @@ test_that("multidimensional normal distribution has correct density", {
 
 test_that("uniform distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::uniform,
+  compare_distribution(
+    greta::uniform,
     stats::dunif,
     parameters = list(min = -2.1, max = -1.2),
     x = runif(100, -2.1, -1.2)
@@ -37,9 +38,9 @@ test_that("uniform distribution has correct density", {
 
 test_that("lognormal distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::lognormal,
+  compare_distribution(
+    greta::lognormal,
     stats::dlnorm,
     parameters = list(meanlog = 1, sdlog = 3),
     x = rlnorm(100, 1, 3)
@@ -48,9 +49,9 @@ test_that("lognormal distribution has correct density", {
 
 test_that("bernoulli distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::bernoulli,
+  compare_distribution(
+    greta::bernoulli,
     extraDistr::dbern,
     parameters = list(prob = 0.3),
     x = rbinom(100, 1, 0.3)
@@ -59,9 +60,9 @@ test_that("bernoulli distribution has correct density", {
 
 test_that("binomial distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::binomial,
+  compare_distribution(
+    greta::binomial,
     stats::dbinom,
     parameters = list(size = 10, prob = 0.8),
     x = rbinom(100, 10, 0.8)
@@ -70,9 +71,9 @@ test_that("binomial distribution has correct density", {
 
 test_that("beta-binomial distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::beta_binomial,
+  compare_distribution(
+    greta::beta_binomial,
     extraDistr::dbbinom,
     parameters = list(
       size = 10,
@@ -85,9 +86,9 @@ test_that("beta-binomial distribution has correct density", {
 
 test_that("negative binomial distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::negative_binomial,
+  compare_distribution(
+    greta::negative_binomial,
     stats::dnbinom,
     parameters = list(size = 3.3, prob = 0.2),
     x = rnbinom(100, 3.3, 0.2)
@@ -96,9 +97,9 @@ test_that("negative binomial distribution has correct density", {
 
 test_that("hypergeometric distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::hypergeometric,
+  compare_distribution(
+    greta::hypergeometric,
     stats::dhyper,
     parameters = list(m = 11, n = 8, k = 5),
     x = rhyper(100, 11, 8, 5)
@@ -107,9 +108,9 @@ test_that("hypergeometric distribution has correct density", {
 
 test_that("poisson distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::poisson,
+  compare_distribution(
+    greta::poisson,
     stats::dpois,
     parameters = list(lambda = 17.2),
     x = rpois(100, 17.2)
@@ -118,21 +119,20 @@ test_that("poisson distribution has correct density", {
 
 test_that("gamma distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::gamma,
+  compare_distribution(
+    greta::gamma,
     stats::dgamma,
     parameters = list(shape = 1.2, rate = 2.3),
     x = rgamma(100, 1.2, 2.3)
   )
 })
 
-
 test_that("inverse gamma distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::inverse_gamma,
+  compare_distribution(
+    greta::inverse_gamma,
     extraDistr::dinvgamma,
     parameters = list(alpha = 1.2, beta = 0.9),
     x = extraDistr::rinvgamma(100, 1.2, 0.9)
@@ -141,9 +141,9 @@ test_that("inverse gamma distribution has correct density", {
 
 test_that("weibull distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::weibull,
+  compare_distribution(
+    greta::weibull,
     dweibull,
     parameters = list(
       shape = 1.2,
@@ -155,9 +155,9 @@ test_that("weibull distribution has correct density", {
 
 test_that("exponential distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::exponential,
+  compare_distribution(
+    greta::exponential,
     stats::dexp,
     parameters = list(rate = 1.9),
     x = rexp(100, 1.9)
@@ -166,9 +166,9 @@ test_that("exponential distribution has correct density", {
 
 test_that("pareto distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::pareto,
+  compare_distribution(
+    greta::pareto,
     extraDistr::dpareto,
     parameters = list(a = 1.9, b = 2.3),
     x = extraDistr::rpareto(100, 1.9, 2.3)
@@ -177,7 +177,7 @@ test_that("pareto distribution has correct density", {
 
 test_that("student distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
+
   dstudent <- extraDistr::dlst
 
   compare_distribution(
@@ -194,9 +194,9 @@ test_that("student distribution has correct density", {
 
 test_that("laplace distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::laplace,
+  compare_distribution(
+    greta::laplace,
     extraDistr::dlaplace,
     parameters = list(mu = -0.9, sigma = 2),
     x = extraDistr::rlaplace(100, -0.9, 2)
@@ -205,9 +205,9 @@ test_that("laplace distribution has correct density", {
 
 test_that("beta distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::beta,
+  compare_distribution(
+    greta::beta,
     stats::dbeta,
     parameters = list(
       shape1 = 2.3,
@@ -219,9 +219,9 @@ test_that("beta distribution has correct density", {
 
 test_that("cauchy distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::cauchy,
+  compare_distribution(
+    greta::cauchy,
     stats::dcauchy,
     parameters = list(
       location = -1.3,
@@ -233,9 +233,9 @@ test_that("cauchy distribution has correct density", {
 
 test_that("logistic distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::logistic,
+  compare_distribution(
+    greta::logistic,
     stats::dlogis,
     parameters = list(
       location = -1.3,
@@ -247,9 +247,9 @@ test_that("logistic distribution has correct density", {
 
 test_that("f distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::f,
+  compare_distribution(
+    greta::f,
     df,
     parameters = list(df1 = 5.9, df2 = 2),
     x = rf(100, 5.9, 2)
@@ -258,9 +258,9 @@ test_that("f distribution has correct density", {
 
 test_that("chi squared distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  compare_distribution(greta::chi_squared,
+  compare_distribution(
+    greta::chi_squared,
     stats::dchisq,
     parameters = list(df = 9.3),
     x = rchisq(100, 9.3)
@@ -269,7 +269,6 @@ test_that("chi squared distribution has correct density", {
 
 test_that("multivariate normal distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   # parameters to test
   m <- 5
@@ -281,7 +280,8 @@ test_that("multivariate normal distribution has correct density", {
     mvtnorm::dmvnorm(x = x, mean = mean, sigma = Sigma, log = log)
   }
 
-  compare_distribution(greta::multivariate_normal,
+  compare_distribution(
+    greta::multivariate_normal,
     dmvnorm2,
     parameters = list(mean = mn, Sigma = sig),
     x = mvtnorm::rmvnorm(100, mn, sig),
@@ -289,83 +289,23 @@ test_that("multivariate normal distribution has correct density", {
   )
 })
 
-test_that("Wishart distribution has correct density", {
+test_that("Wishart and LKJ distributions have correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
-  # parameters to test
-  m <- 5
-  df <- m + 1
-  sig <- rWishart(1, df, diag(m))[, , 1]
+  # NOTE: we no longer have unit tests for the densities of the Wishart and LKJ
+  # distributions, because the greta implementation uses the distribution over
+  # hcolesky factors, combined with the log det jacobian for change of support
+  # from the cholesky factor to the symmetric matrix. This distribution is
+  # non-standard, so there is no reference distribution in R against which to
+  # compare the log probs. Instead, the implied densities for these
+  # distributions are tested using integration tests with the MCMC sampler, in
+  # test_posteriors_wishart.R and test_posteriors_lkj.R.
+  skip()
 
-  # wrapper for argument names
-  dwishart <- function(x, df, Sigma, log = FALSE) { # nolint
-    ans <- MCMCpack::dwish(W = x, v = df, S = Sigma)
-    if (log) {
-      ans <- log(ans)
-    }
-    ans
-  }
-
-  # no vectorised wishart, so loop through all of these
-  replicate(
-    10,
-    compare_distribution(greta::wishart,
-      dwishart,
-      parameters = list(
-        df = df,
-        Sigma = sig
-      ),
-      x = rWishart(1, df, sig)[, , 1],
-      multivariate = TRUE
-    )
-  )
-})
-
-test_that("lkj distribution has correct density", {
-  skip_if_not(check_tf_version())
-  skip_on_cran()
-
-  # parameters to test
-  m <- 5
-  eta <- 3
-
-  # normalising component of lkj (depends only on eta and dimension)
-  lkj_log_normalising <- function(eta, n) {
-    log_pi <- log(pi)
-    ans <- 0
-    for (k in 1:(n - 1)) {
-      ans <- ans + log_pi * (k / 2)
-      ans <- ans + lgamma(eta + (n - 1 - k) / 2)
-      ans <- ans - lgamma(eta + (n - 1) / 2)
-    }
-    ans
-  }
-
-  # lkj density
-  dlkj_correlation <- function(x, eta, log = FALSE, dimension = NULL) {
-    res <- (eta - 1) * log(det(x)) - lkj_log_normalising(eta, ncol(x))
-    if (!log) {
-      res <- exp(res)
-    }
-    res
-  }
-
-  # no vectorised lkj, so loop through all of these
-  replicate(
-    10,
-    compare_distribution(greta::lkj_correlation,
-      dlkj_correlation,
-      parameters = list(eta = eta, dimension = m),
-      x = rlkjcorr(1, eta = 1, dimension = m),
-      multivariate = TRUE
-    )
-  )
 })
 
 test_that("multinomial distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   # parameters to test
   m <- 5
@@ -377,7 +317,8 @@ test_that("multinomial distribution has correct density", {
     apply(x, 1, stats::dmultinom, size = size, prob = prob)
   }
 
-  compare_distribution(greta::multinomial,
+  compare_distribution(
+    greta::multinomial,
     dmultinom_vec,
     parameters = list(
       size = size,
@@ -390,7 +331,6 @@ test_that("multinomial distribution has correct density", {
 
 test_that("categorical distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   # parameters to test
   m <- 5
@@ -401,7 +341,8 @@ test_that("categorical distribution has correct density", {
     apply(x, 1, stats::dmultinom, size = 1, prob = prob)
   }
 
-  compare_distribution(greta::categorical,
+  compare_distribution(
+    greta::categorical,
     dcategorical_vec,
     parameters = list(prob = prob),
     x = t(rmultinom(100, 1, prob)),
@@ -410,8 +351,7 @@ test_that("categorical distribution has correct density", {
 })
 
 test_that("dirichlet distribution has correct density", {
-  # skip_if_not(check_tf_version())
-  skip_on_cran()
+  skip_if_not(check_tf_version())
   # parameters to test
   m <- 5
   alpha <- t(runif(m))
@@ -427,14 +367,14 @@ test_that("dirichlet distribution has correct density", {
 
 test_that("dirichlet-multinomial distribution has correct density", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   # parameters to test
   size <- 10
   m <- 5
   alpha <- t(runif(m))
 
-  compare_distribution(greta::dirichlet_multinomial,
+  compare_distribution(
+    greta::dirichlet_multinomial,
     extraDistr::ddirmnom,
     parameters = list(
       size = size,
@@ -451,7 +391,6 @@ test_that("dirichlet-multinomial distribution has correct density", {
 
 test_that("scalar-valued distributions can be defined in models", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   x <- randn(5)
   y <- round(randu(5))
@@ -527,7 +466,6 @@ test_that("scalar-valued distributions can be defined in models", {
 
 test_that("array-valued distributions can be defined in models", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   dim <- c(5, 2)
   x <- randn(5, 2)
@@ -609,7 +547,6 @@ test_that("array-valued distributions can be defined in models", {
 
 test_that("distributions can be sampled from by MCMC", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   x <- randn(100)
   y <- round(randu(100))
@@ -701,55 +638,57 @@ test_that("distributions can be sampled from by MCMC", {
 
 test_that("uniform distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_ci()
-  skip_on_cran()
+  # skip_on_ci()
 
   # bad types
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     uniform(min = 0, max = NA)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     uniform(min = 0, max = head)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     uniform(min = 1:3, max = 5)
   )
 
   # good types, bad values
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     uniform(min = -Inf, max = Inf)
   )
 
   # lower not below upper
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     uniform(min = 1, max = 1)
   )
-
 })
 
 test_that("poisson() and binomial() error informatively in glm", {
-  skip_on_ci()
+  # skip_on_ci()
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   # if passed as an object
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     glm(1 ~ 1, family = poisson)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     glm(1 ~ 1, family = binomial)
   )
 
   # if executed alone
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     glm(1 ~ 1, family = poisson())
   )
 
   # if given a link
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     glm(1 ~ 1, family = poisson("sqrt"))
   )
 })
@@ -757,7 +696,6 @@ test_that("poisson() and binomial() error informatively in glm", {
 test_that("wishart distribution errors informatively", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(3, 3)
   b <- randn(3, 3, 3)
   c <- randn(3, 2)
@@ -767,20 +705,19 @@ test_that("wishart distribution errors informatively", {
     "greta_array"
   ))
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     wishart(3, b)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     wishart(3, c)
   )
-
 })
 
-
 test_that("lkj_correlation distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   dim <- 3
 
@@ -789,34 +726,33 @@ test_that("lkj_correlation distribution errors informatively", {
     "greta_array"
   ))
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     lkj_correlation(-1, dim)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     lkj_correlation(c(3, 3), dim)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     lkj_correlation(uniform(0, 1, dim = 2), dim)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     lkj_correlation(4, dimension = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     lkj_correlation(4, dim = c(3, 3))
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     lkj_correlation(4, dim = NA)
   )
 })
 
 test_that("multivariate_normal distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   m_a <- randn(1, 3)
   m_b <- randn(2, 3)
@@ -840,11 +776,11 @@ test_that("multivariate_normal distribution errors informatively", {
   ))
 
   # bad means
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_c, a)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_d, a)
   )
 
@@ -855,46 +791,45 @@ test_that("multivariate_normal distribution errors informatively", {
   ))
 
   # bad sigmas
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, b)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, c)
   )
 
   # mismatched parameters
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, d)
   )
 
   # scalars
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(0, 1)
   )
 
   # bad n_realisations
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, a, n_realisations = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, a, n_realisations = c(1, 3))
   )
 
   # bad dimension
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, a, dimension = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multivariate_normal(m_a, a, dimension = c(1, 3))
   )
 })
 
 test_that("multinomial distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   p_a <- randu(1, 3)
   p_b <- randu(2, 3)
@@ -923,32 +858,31 @@ test_that("multinomial distribution errors informatively", {
   ))
 
   # scalars
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multinomial(c(1), 1)
   )
 
   # bad n_realisations
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multinomial(10, p_a, n_realisations = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multinomial(10, p_a, n_realisations = c(1, 3))
   )
 
   # bad dimension
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multinomial(10, p_a, dimension = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     multinomial(10, p_a, dimension = c(1, 3))
   )
 })
 
 test_that("categorical distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   p_a <- randu(1, 3)
   p_b <- randu(2, 3)
@@ -965,32 +899,31 @@ test_that("categorical distribution errors informatively", {
   ))
 
   # scalars
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     categorical(1),
   )
 
   # bad n_realisations
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     categorical(p_a, n_realisations = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     categorical(p_a, n_realisations = c(1, 3))
   )
 
   # bad dimension
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     categorical(p_a, dimension = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     categorical(p_a, dimension = c(1, 3))
   )
 })
 
 test_that("dirichlet distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   alpha_a <- randu(1, 3)
   alpha_b <- randu(2, 3)
@@ -1008,25 +941,25 @@ test_that("dirichlet distribution errors informatively", {
   ))
 
   # scalars
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet(1),
   )
 
   # bad n_realisations
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet(alpha_a, n_realisations = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet(alpha_a, n_realisations = c(1, 3))
   )
 
   # bad dimension
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet(alpha_a, dimension = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet(alpha_a, dimension = c(1, 3))
   )
 })
@@ -1034,7 +967,6 @@ test_that("dirichlet distribution errors informatively", {
 
 test_that("dirichlet values sum to one", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   alpha <- uniform(0, 10, dim = c(1, 5))
   x <- dirichlet(alpha)
@@ -1047,12 +979,10 @@ test_that("dirichlet values sum to one", {
 
 test_that("dirichlet-multinomial distribution errors informatively", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
 
   alpha_a <- randu(1, 3)
   alpha_b <- randu(2, 3)
 
-
   # same size & probs
   expect_true(inherits(
     dirichlet_multinomial(size = 10, alpha_a),
@@ -1077,42 +1007,32 @@ test_that("dirichlet-multinomial distribution errors informatively", {
   ))
 
   # scalars
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet_multinomial(c(1), 1)
   )
 
   # bad n_realisations
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet_multinomial(10, alpha_a, n_realisations = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet_multinomial(10, alpha_a, n_realisations = c(1, 3))
   )
 
   # bad dimension
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet_multinomial(10, alpha_a, dimension = -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dirichlet_multinomial(10, alpha_a, dimension = c(1, 3))
   )
 })
 
-test_that("Wishart can use a choleskied Sigma", {
-  skip_if_not(check_tf_version())
-  skip_on_cran()
-
-  sig <- lkj_correlation(2, dim = 2)
-  w <- wishart(5, sig)
-  m <- model(w, precision = "double")
-  expect_ok(draws <- mcmc(m, warmup = 0, n_samples = 5, verbose = FALSE))
-})
-
 test_that("multivariate distribs with matrix params can be sampled from", {
   skip_if_not(check_tf_version())
-  skip_on_cran()
+
   n <- 10
   k <- 3
 
