@@ -1,7 +1,6 @@
 test_that("extract works like R", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(10)
   b <- randn(10, 1)
   c <- randn(10, 5)
@@ -57,7 +56,6 @@ test_that("extract works like R", {
 test_that("replace works like R", {
   skip_if_not(check_tf_version())
 
-
   # check using expressions, and comparing the whole object to which replacement
   # was applied
 
@@ -302,7 +300,6 @@ test_that("replace works like R", {
 test_that("rep works like R", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(10)
   b <- randn(10, 1)
   c <- randn(10, 5)
@@ -348,7 +345,6 @@ test_that("rep works like R", {
 test_that("rbind, cbind and c work like R", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(5, 1)
   b <- randn(1, 5)
   d <- randn(5, 5)
@@ -375,7 +371,6 @@ test_that("rbind, cbind and c work like R", {
 test_that("abind works like R", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(5, 1, 3)
   b <- randn(1, 1, 3)
   c <- randn(5, 1, 1)
@@ -389,16 +384,15 @@ test_that("abind works like R", {
 test_that("abind errors informatively", {
   skip_if_not(check_tf_version())
 
-
   a <- ones(5, 1, 3)
   b <- ones(1, 1, 3)
   c <- ones(5, 1, 1)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     abind(a, b)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     abind(a, c, along = 5)
   )
 
@@ -407,7 +401,6 @@ test_that("abind errors informatively", {
 test_that("rbind and cbind can prepend R arrays to greta arrays", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(5, 1)
   b <- ones(5, 1)
 
@@ -431,9 +424,8 @@ test_that("rbind and cbind can prepend R arrays to greta arrays", {
 test_that("assign errors on variable greta arrays", {
   skip_if_not(check_tf_version())
 
-
   z <- normal(0, 1, dim = 5)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     z[1] <- 3
   )
 })
@@ -441,15 +433,14 @@ test_that("assign errors on variable greta arrays", {
 test_that("rbind and cbind give informative error messages", {
   skip_if_not(check_tf_version())
 
-
   a <- as_data(randn(5, 1))
   b <- as_data(randn(1, 5))
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     rbind(a, b)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     cbind(a, b)
   )
 })
@@ -457,18 +448,17 @@ test_that("rbind and cbind give informative error messages", {
 test_that("replacement gives informative error messages", {
   skip_if_not(check_tf_version())
 
-
   x <- ones(2, 2, 2)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     x[1:2, , 1] <- seq_len(3)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     x[1, 1, 3] <- 1
   )
 
   x <- ones(2)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     x[3] <- 1
   )
 })
@@ -476,14 +466,13 @@ test_that("replacement gives informative error messages", {
 test_that("extraction gives informative error messages", {
   skip_if_not(check_tf_version())
 
-
   x <- ones(2, 2, 2)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     x[1, 1, 3]
   )
 
   x <- ones(2)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     x[3]
   )
 })
@@ -491,7 +480,6 @@ test_that("extraction gives informative error messages", {
 test_that("stochastic and operation greta arrays can be extracted", {
   skip_if_not(check_tf_version())
 
-
   a <- normal(0, 1, dim = c(3, 4))
   a_sub <- a[1:2, 2:3]
   expect_identical(dim(a_sub), c(2L, 2L))
@@ -504,7 +492,6 @@ test_that("stochastic and operation greta arrays can be extracted", {
 test_that("extract, replace, combine work in models", {
   skip_if_not(check_tf_version())
 
-
   # extract
   a <- normal(0, 1, dim = c(3, 4))
   a_sub <- a[1:2, 2:3]
@@ -530,7 +517,6 @@ test_that("extract, replace, combine work in models", {
 test_that("length and dim work", {
   skip_if_not(check_tf_version())
 
-
   ga_data <- as_data(matrix(1:9, nrow = 3))
   ga_stochastic <- normal(0, 1, dim = c(3, 3))
   ga_operation <- ga_data * ga_stochastic
@@ -549,7 +535,6 @@ test_that("length and dim work", {
 test_that("dim<- works", {
   skip_if_not(check_tf_version())
 
-
   x <- randn(3, 4, 2)
 
   new_dim <- c(2, 2, 6)
@@ -565,7 +550,6 @@ test_that("dim<- works", {
 test_that("greta_array() reshapes array-like greta arrays like array", {
   skip_if_not(check_tf_version())
 
-
   x_ <- randu(3, 4, 2)
   x <- as_data(x_)
   new_dim <- c(12, 2)
@@ -592,7 +576,6 @@ test_that("greta_array() reshapes array-like greta arrays like array", {
 test_that("greta_array() reshapes scalar greta arrays like array", {
   skip_if_not(check_tf_version())
 
-
   x_ <- randu(1)
   x <- as_data(x_)
   new_dim <- c(12, 2)
@@ -618,22 +601,21 @@ test_that("greta_array() reshapes scalar greta arrays like array", {
 test_that("dim<- errors as expected", {
   skip_if_not(check_tf_version())
 
-
   x <- zeros(3, 4)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dim(x) <- pi[0]
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dim(x) <- c(1, NA)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dim(x) <- c(1, -1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     dim(x) <- 13
   )
 })
@@ -641,7 +623,6 @@ test_that("dim<- errors as expected", {
 test_that("dim<- works in a model", {
   skip_if_not(check_tf_version())
 
-
   y <- rnorm(5)
 
   x1 <- greta_array(1:12, c(3, 4))
@@ -664,7 +645,6 @@ test_that("dim<- works in a model", {
 test_that("c handles NULLs and lists", {
   skip_if_not(check_tf_version())
 
-
   x <- normal(0, 1)
   y <- as_data(3:1)
 
