@@ -1,7 +1,3 @@
-if (check_tf_version()) {
-  tensorflow::tf$compat$v1$reset_default_graph()
-}
-
 set.seed(2020 - 02 - 11)
 
 test_that("simple functions work as expected", {
@@ -69,12 +65,11 @@ test_that("primitive functions work as expected", {
 test_that("cummax and cummin functions error informatively", {
   skip_if_not(check_tf_version())
 
-
   cumulative_funs <- list(cummax, cummin)
   x <- as_data(randn(10))
 
   for (fun in cumulative_funs) {
-    expect_snapshot_error(
+    expect_snapshot(error = TRUE,
       fun(x)
     )
   }
@@ -83,12 +78,11 @@ test_that("cummax and cummin functions error informatively", {
 test_that("complex number functions error informatively", {
   skip_if_not(check_tf_version())
 
-
   complex_funs <- list(Im, Re, Arg, Conj, Mod)
   x <- as_data(randn(25, 4))
 
   for (fun in complex_funs) {
-    expect_snapshot_error(
+    expect_snapshot(error = TRUE,
       fun(x)
     )
   }
@@ -149,7 +143,6 @@ test_that("kronecker works with greta and base array arguments", {
 test_that("aperm works as expected", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(5, 4, 3, 2, 1)
 
   # default is to reverse dims
@@ -205,7 +198,6 @@ test_that("cumulative functions work as expected", {
 test_that("apply works as expected", {
   skip_if_not(check_tf_version())
 
-
   # check apply.greta_array works like R's apply for X
   check_apply <- function(X, MARGIN, FUN) { # nolint
     check_op(apply, a,
@@ -248,24 +240,23 @@ test_that("tapply works as expected", {
 test_that("cumulative functions error as expected", {
   skip_if_not(check_tf_version())
 
-
   a <- as_data(randn(1, 5))
   b <- as_data(randn(5, 1, 1))
 
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     cumsum(a)
     )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     cumsum(b)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     cumprod(a)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     cumprod(b)
   )
 
@@ -274,7 +265,6 @@ test_that("cumulative functions error as expected", {
 test_that("sweep works as expected", {
   skip_if_not(check_tf_version())
 
-
   stats_list <- list(randn(5), randn(25))
   x <- randn(5, 25)
 
@@ -295,7 +285,6 @@ test_that("sweep works as expected", {
 test_that("sweep works for numeric x and greta array STATS", {
   skip_if_not(check_tf_version())
 
-
   stats <- randn(5)
   ga_stats <- as_data(stats)
   x <- randn(5, 25)
@@ -309,7 +298,6 @@ test_that("sweep works for numeric x and greta array STATS", {
 test_that("solve and sweep and kronecker error as expected", {
   skip_if_not(check_tf_version())
 
-
   a <- as_data(randn(5, 25))
   b <- as_data(randn(5, 25, 2))
   c <- as_data(randn(5, 5))
@@ -318,62 +306,74 @@ test_that("solve and sweep and kronecker error as expected", {
   # solve
 
   # a must be 2D
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     solve(b, a)
   )
 
   # b must also be 2D
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     solve(c, b)
   )
 
   # only square matrices allowed for first element
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     solve(a, a)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     solve(a)
   )
 
   # dimension of second array must match
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     solve(c, t(a))
   )
 
   # sweep
   # x must be 2D
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     sweep(b, 1, stats)
   )
 
   # dim must be either 1 or 2
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     sweep(a, 3, stats)
   )
 
   # stats must have the correct number of elements
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     sweep(a, 1, c(stats, stats))
   )
 
   # stats must be a column vector
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     sweep(a, 1, t(stats))
   )
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     sweep(a, 2, stats)
   )
 
   # kronecker
   # X must be 2D
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     kronecker(a, b)
   )
 
   # Y must be 2D
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     kronecker(b, c)
   )
 
@@ -384,19 +384,19 @@ test_that("colSums etc. error as expected", {
 
   x <- as_data(randn(3, 4, 5))
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     colSums(x, dims = 3)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     rowSums(x, dims = 3)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     colMeans(x, dims = 3)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     rowMeans(x, dims = 3)
   )
 
@@ -409,19 +409,19 @@ test_that("forwardsolve and backsolve error as expected", {
   b <- as_data(randn(5, 25))
   c <- chol(a)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     forwardsolve(a, b, k = 1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     backsolve(a, b, k = 1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     forwardsolve(a, b, transpose = TRUE)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     backsolve(a, b, transpose = TRUE)
   )
 
@@ -430,18 +430,17 @@ test_that("forwardsolve and backsolve error as expected", {
 test_that("tapply errors as expected", {
   skip_if_not(check_tf_version())
 
-
   group <- sample.int(5, 10, replace = TRUE)
   a <- ones(10, 1)
   b <- ones(10, 2)
 
   # X must be a column vector
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     tapply(b, group, "sum")
   )
 
   # INDEX can't be a greta array
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     tapply(a, as_data(group), "sum")
   )
 })
@@ -449,7 +448,6 @@ test_that("tapply errors as expected", {
 test_that("eigen works as expected", {
   skip_if_not(check_tf_version())
 
-
   k <- 4
   x <- rWishart(1, k + 1, diag(k))[, , 1]
   x_ga <- as_data(x)
@@ -460,7 +458,6 @@ test_that("eigen works as expected", {
   r_out_vals <- eigen(x, only.values = TRUE)
   greta_out_vals <- eigen(as_data(x), only.values = TRUE)
 
-
   # values
   compare_op(r_out$values, grab(greta_out$values))
 
@@ -498,7 +495,7 @@ test_that("ignored options are errored/warned about", {
   skip_if_not(check_tf_version())
 
   x <- ones(3, 3)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     round(x, 2)
   )
 
@@ -526,39 +523,39 @@ test_that("incorrect dimensions are errored about", {
   x <- ones(3, 3, 3)
   y <- ones(3, 4)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     t(x)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     aperm(x, 2:1)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     chol(x)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     chol(y)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     chol2symm(x)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     chol2symm(y)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     eigen(x)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     eigen(y)
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     rdist(x, y)
   )
 })
