@@ -1,7 +1,3 @@
-if (check_tf_version()) {
-  tensorflow::tf$compat$v1$reset_default_graph()
-}
-
 set.seed(2020 - 02 - 11)
 
 test_that("head and tail work", {
