@@ -13,7 +13,6 @@ test_that("continuous joint variables can be sampled from", {
 test_that("truncated continuous joint variables can be sampled from", {
   skip_if_not(check_tf_version())
 
-
   x <- joint(
     normal(0, 1, truncation = c(0, Inf)),
     normal(0, 2, truncation = c(0, Inf)),
@@ -26,7 +25,6 @@ test_that("truncated continuous joint variables can be sampled from", {
 test_that("uniform joint variables can be sampled from", {
   skip_if_not(check_tf_version())
 
-
   x <- joint(
     uniform(0, 1),
     uniform(0, 2),
@@ -39,7 +37,6 @@ test_that("uniform joint variables can be sampled from", {
 test_that("joint normals with different truncation types can be sampled", {
   skip_if_not(check_tf_version())
 
-
   x <- joint(
     normal(0, 1, truncation = c(0, Inf)),
     normal(0, 2, truncation = c(-Inf, 0)),
@@ -52,7 +49,6 @@ test_that("joint normals with different truncation types can be sampled", {
 test_that("fixed continuous joint distributions can be sampled from", {
   skip_if_not(check_tf_version())
 
-
   obs <- matrix(rnorm(3, 0, 2), 100, 3)
   mu <- variable(dim = 3)
   distribution(obs) <- joint(normal(mu[1], 1),
@@ -67,10 +63,10 @@ test_that("fixed continuous joint distributions can be sampled from", {
 test_that("fixed discrete joint distributions can be sampled from", {
   skip_if_not(check_tf_version())
 
-
   obs <- matrix(rbinom(300, 1, 0.5), 100, 3)
   probs <- variable(0, 1, dim = 3)
-  distribution(obs) <- joint(bernoulli(probs[1]),
+  distribution(obs) <- joint(
+    bernoulli(probs[1]),
     bernoulli(probs[2]),
     bernoulli(probs[3]),
     dim = 100
@@ -82,8 +78,7 @@ test_that("fixed discrete joint distributions can be sampled from", {
 test_that("joint of fixed and continuous distributions errors", {
   skip_if_not(check_tf_version())
 
-
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     joint(
       bernoulli(0.5),
       normal(0, 1)
@@ -94,12 +89,11 @@ test_that("joint of fixed and continuous distributions errors", {
 test_that("joint with insufficient distributions errors", {
   skip_if_not(check_tf_version())
 
-
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     joint(normal(0, 2))
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     joint()
   )
 })
@@ -107,8 +101,7 @@ test_that("joint with insufficient distributions errors", {
 test_that("joint with non-scalar distributions errors", {
   skip_if_not(check_tf_version())
 
-
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     joint(
       normal(0, 2, dim = 3),
       normal(0, 1, dim = 3)
@@ -119,7 +112,6 @@ test_that("joint with non-scalar distributions errors", {
 test_that("joint of normals has correct density", {
   skip_if_not(check_tf_version())
 
-
   joint_greta <- function(means, sds, dim) {
     joint(normal(means[1], sds[1]),
       normal(means[2], sds[2]),
@@ -155,7 +147,6 @@ test_that("joint of normals has correct density", {
 test_that("joint of truncated normals has correct density", {
   skip_if_not(check_tf_version())
 
-
   joint_greta <- function(means, sds, lower, upper, dim) {
     joint(normal(means[1], sds[1], truncation = c(lower[1], upper[1])),
       normal(means[2], sds[2], truncation = c(lower[2], upper[2])),
@@ -205,7 +196,6 @@ test_that("joint of truncated normals has correct density", {
 test_that("joint of uniforms has correct density", {
   skip_if_not(check_tf_version())
 
-
   joint_greta <- function(lower, upper, dim) {
     joint(uniform(lower[1], upper[1]),
       uniform(lower[2], upper[2]),
@@ -246,7 +236,6 @@ test_that("joint of uniforms has correct density", {
 test_that("joint of Poissons has correct density", {
   skip_if_not(check_tf_version())
 
-
   joint_greta <- function(rates, dim) {
     joint(poisson(rates[1]),
       poisson(rates[2]),
