@@ -1,6 +1,5 @@
 test_that("`distribution<-` works in models", {
   skip_if_not(check_tf_version())
-  
 
   # with a distribution parameter
   y <- as_data(randn(5))
@@ -14,7 +13,6 @@ test_that("`distribution<-` works in models", {
 
 test_that("distribution() works", {
   skip_if_not(check_tf_version())
-  
 
   a <- normal(0, 1)
   x <- as_data(randn(5))
@@ -33,34 +31,33 @@ test_that("distribution() works", {
 
 test_that("`distribution<-` errors informatively", {
   skip_if_not(check_tf_version())
-  
 
   y <- randn(3, 3, 2)
   x <- randn(1)
 
   # not a greta array with a distribution on the right
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(y) <- x
   )
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(y) <- as_data(x)
   )
 
   # no density on the right
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(y) <- variable()
   )
 
   # non-scalar and wrong dimensions
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(y) <- normal(0, 1, dim = c(3, 3, 1))
   )
 
   # double assignment of distribution to node
   y_ <- as_data(y)
   distribution(y_) <- normal(0, 1)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(y_) <- normal(0, 1)
   )
 
@@ -69,25 +66,25 @@ test_that("`distribution<-` errors informatively", {
   y2 <- as_data(y)
   d <- normal(0, 1)
   distribution(y1) <- d
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(y2) <- y1
   )
 
   # assignment to a variable
   z <- variable()
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(z) <- normal(0, 1)
   )
 
   # assignment to an op
   z2 <- z^2
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(z2) <- normal(0, 1)
   )
 
   # assignment to another distribution
   u <- uniform(0, 1)
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     distribution(z2) <- normal(0, 1)
   )
 
@@ -95,11 +92,11 @@ test_that("`distribution<-` errors informatively", {
 
 test_that("distribution() errors informatively", {
   skip_if_not(check_tf_version())
-  
 
   y <- randn(3)
 
-  expect_snapshot_error(
+  expect_snapshot(
+    error = TRUE,
     distribution(y)
   )
 })
