@@ -1,10 +1,8 @@
 test_that("transformations work as expected", {
   skip_if_not(check_tf_version())
 
-
   a <- randn(25, 4)
 
-
   r_icloglog <- function(x) {
     1 - exp(-1 * exp(x))
   }
@@ -31,10 +29,9 @@ test_that("transformations work as expected", {
 test_that("imultilogit errors informatively", {
   skip_if_not(check_tf_version())
 
-
   x <- ones(3, 4, 3)
 
-  expect_snapshot_error(
+  expect_snapshot(error = TRUE,
     imultilogit(x)
   )
 
